##############################################################################
# Copyright (C) 2008 Novell Inc. All rights reserved.
# Copyright (C) 2008 SUSE Linux Products GmbH. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#   - Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#   - Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
# 
#   - Neither the name of Novell Inc. nor of SUSE Linux Products GmbH nor the
#     names of its contributors may be used to endorse or promote products
#     derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL Novell Inc. OR SUSE Linux Products GmbH OR
# THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##############################################################################

# Author: Bart Whiteley <bwhiteley suse.de>

from pywbem.cim_provider2 import ProviderProxy
import pywbem
import types
import syslog
import sys


import cmpi

##==============================================================================
##
## _exception_to_error()
##
##     This function converts a cmpi.CMPIException to a pywbem.CIMError.
##
##==============================================================================

def _exception_to_error(ex):

    code = ex.get_error_code()
    desc = ex.get_description()

    if code < 0 or code > 17:
        if desc is None:
            desc = str(code)
        else:
            desc = str(code) + ':' + desc
        code = pywbem.CIM_ERR_FAILED

    return pywbem.CIMError(code, desc)

##==============================================================================
##
## ExceptionMethodWrapper
##
##     This class puts an exception translation block around any method. This
##     block catches a cmpi.CMPIException, converts it a pywbem.CIMError, and
##     raises the new exception.
##
##==============================================================================

class ExceptionMethodWrapper:

    def __init__(self, meth):
        self.meth = meth

    def __call__(self, *args, **kwds):
        try:
            return self.meth(*args, **kwds)
        except cmpi.CMPIException,e:
            exc_class, exc, tb = sys.exc_info()
            new_exc = _exception_to_error(e)
            raise new_exc.__class__, new_exc, tb


##==============================================================================
##
## ExceptionClassWrapper
##
##     This class puts an exception translation block around all methods of any
##     class. It creates an ExceptionMethodWrapper to invoke each method. For
##     example, the following snipett wraps an instance of the Gadget class.
##    
##         g = Gadget()
##         w = ExceptionClassWrapper(g)
##         w.foo() # call g.foo() with exception translation block around it.
##
##==============================================================================

class ExceptionClassWrapper:

    def __init__(self, obj):
        self.obj = obj

    def __getattr__(self, name):
        attr = getattr(self.obj, name)

        if type(attr) is types.MethodType:
            return ExceptionMethodWrapper(attr)
        else:
            return attr

##==============================================================================
##
## _mwrap()
##
##     Wrap a method in a try block.
##
##==============================================================================

def _mwrap(obj, meth, *args, **kwds):
    try:
        return obj.meth(*args, **kwds)
    except cmpi.CMPIException,e:
        raise _exception_to_error(e)

##==============================================================================
##
## _fwrap()
##
##     Wrap a function in a try block.
##
##==============================================================================

def _fwrap(meth, *args, **kwds):
    try:
        return meth(*args, **kwds)
    except cmpi.CMPIException,e:
        raise _exception_to_error(e)

##==============================================================================
##
##
##
##==============================================================================

class ContextWrap(object):
    def __init__(self, proxy, cmpicontext):
        self.proxy = proxy
        self.cmpicontext = cmpicontext

    def __getitem__(self, key):
        data = self.cmpicontext.get_entry(key)
        _type, is_array = _cmpi_type2string(data.type)
        return self.proxy.cmpi2pywbem_data(data, _type, is_array)

    def __setitem__(self, key, pval):
        data, _type = self.proxy.pywbem2cmpi_value(pval)
        ctype = _pywbem2cmpi_typemap[_type]
        if isinstance(pval, list):
            ctype = ctype | cmpi.CMPI_ARRAY
        self.cmpicontext.add_entry(str(key), data, ctype)

    def __len__(self):
        return self.cmpicontext.get_entry_count()

    def __repr__(self):
        return `self.todict()`

    def keys(self):
        return self.todict().keys()

    def items(self):
        return self.todict().items()

    def values(self):
        return self.todict().values()

    def __contains__(self, key):
        return key in self.todict()

    def has_key(self, key):
        return self.todict().has_key(key)

    def iterkeys(self):
        return self.todict().iterkeys()

    def itervalues(self):
        return self.todict().itervalues()

    def iteritems(self):
        return self.todict().iteritems()

    def update(self, *args, **kwargs):
        for mapping in args:
            if hasattr(mapping, 'items'):
                for k, v in mapping.items():
                    self[k] = v
            else:
                for (k, v) in mapping:
                    self[k] = v
        for k, v in kwargs.items():
            self[k] = v

    def get(self, key, default = None):
        try:
            return self.todict()[key]
        except KeyError:
            return default

    def todict(self):
        d = {}
        for i in xrange(0, self.cmpicontext.get_entry_count()):
            name, data = self.cmpicontext.get_entry_at(i)
            _type, is_array = _cmpi_type2string(data.type)
            pval = self.proxy.cmpi2pywbem_data(data, _type, is_array)
            d[name] = pval
        return d


class BrokerCIMOMHandle(object):
    def __init__(self, proxy, ctx):
        #self.broker = proxy.broker
        self.broker = ExceptionClassWrapper(proxy.broker)
        self.proxy = proxy
        self.ctx = ctx

    def _yield_instance_names(self, e):
        while e and e.hasNext():
            data=e.next()
            assert(data.type == cmpi.CMPI_ref)
            piname=self.proxy.cmpi2pywbem_instname(data.value.ref)
            yield piname

    def EnumerateInstanceNames(self, ns, cn):
        cop = self.broker.new_object_path(ns, cn)
        e = self.broker.enumInstanceNames(self.ctx, cop)
        while e and e.hasNext():
            data=e.next()
            assert(data.type == cmpi.CMPI_ref)
            piname=self.proxy.cmpi2pywbem_instname(data.value.ref)
            yield piname

    def EnumerateInstances(self, ns, cn, props = None):
        cop = self.broker.new_object_path(ns, cn)
        e = self.broker.enumInstances(self.ctx, cop, props)
        while e and e.hasNext():
            data=e.next()
            assert(data.type == cmpi.CMPI_instance)
            pinst=self.proxy.cmpi2pywbem_inst(data.value.inst)
            yield pinst

    def GetInstance(self, path, props = None):
        cop = self.proxy.pywbem2cmpi_instname(path)
        ci = self.broker.getInstance(self.ctx, cop, props)
        if ci is None:
            return None
        return self.proxy.cmpi2pywbem_inst(ci)

    def Associators(self, path, assocClass = None, resultClass = None, 
        role = None, resultRole = None, props = None):
        cop = self.proxy.pywbem2cmpi_instname(path)
        e = self.broker.associators(self.ctx, cop, assocClass, resultClass,
            role, resultRole, props)
        while e and e.hasNext():
            data = e.next()
            assert(data.type == cmpi.CMPI_instance)
            pinst=self.proxy.cmpi2pywbem_inst(data.value.inst)
            yield pinst

    def AssociatorNames(self, path, assocClass = None, resultClass = None, 
        role = None, resultRole = None):
        cop = self.proxy.pywbem2cmpi_instname(path)
        e = self.broker.associatorNames(self.ctx, cop, assocClass, resultClass,
            role, resultRole)
        while e and e.hasNext():
            data = e.next()
            assert(data.type == cmpi.CMPI_ref)
            piname=self.proxy.cmpi2pywbem_instname(data.value.ref)
            yield piname

    def References(self, path, resultClass=None, role=None, props=None):
        cop = self.proxy.pywbem2cmpi_instname(path)
        e = self.broker.references(self.ctx, cop, resultClass,
            role, props)
        while e and e.hasNext():
            data = e.next()
            assert(data.type == cmpi.CMPI_instance)
            pinst=self.proxy.cmpi2pywbem_inst(data.value.inst)
            yield pinst
            
    def ReferenceNames(self, path, resultClass=None, role=None):
        cop = self.proxy.pywbem2cmpi_instname(path)
        e = self.broker.referenceNames(self.ctx, cop, resultClass, role)
        while e and e.hasNext():
            data = e.next()
            assert(data.type == cmpi.CMPI_ref)
            piname=self.proxy.cmpi2pywbem_instname(data.value.ref)
            yield piname

    def InvokeMethod(self, path, method, **params):
        if not isinstance(path, pywbem.CIMClassName) and \
                not isinstance(path, pywbem.CIMInstanceName):
            # invalid parameter
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER)
        if path.namespace is None:
            # must have namespace
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_NAMESPACE)
        cop = self.proxy.pywbem2cmpi_instname(path)

        # dirty hack to get upcall agrument to correct format,
        # i.e. dictionary of (type, value)
        wparams = {}
        for name, value in params.items():
            if isinstance(value, list):
                data, _type = self.proxy.pywbem2cmpi_value(value[0])
            else:
                data, _type = self.proxy.pywbem2cmpi_value(value)
            wparams[name] = (_type, value)

        inargs=self.proxy.pywbem2cmpi_args(wparams)
        poutargs = self.broker.new_args()
        rc=self.broker.invokeMethod(self.ctx, cop, method, inargs, poutargs)
        outrc = self.proxy.cmpi2pywbem_data(rc)
        outargs = self.proxy.cmpi2pywbem_args(poutargs)
        rslt = (outrc,outargs)
        return rslt
        
    def CreateInstance(self, instance):
        if instance.path is None or not instance.path:
            # no INVALID_PATH error... INVALID_NAMESPACE is best option
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_NAMESPACE)
        if instance.path.namespace is None or not instance.path.namespace:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_NAMESPACE)
        cop = self.proxy.pywbem2cmpi_instname(instance.path)
        inst = self.proxy.pywbem2cmpi_inst(instance)
        ciname = self.broker.createInstance(self.ctx, cop, inst)
        if ciname is None:
            return None
        return self.proxy.cmpi2pywbem_instname(ciname)
    
    def DeleteInstance(self, path):
        cop = self.proxy.pywbem2cmpi_instname(path)
        return self.broker.deleteInstance(self.ctx, cop)

    def ModifyInstance(self, instance):
        if instance.path is None or not instance.path:
            # no INVALID_PATH error... INVALID_NAMESPACE is best option
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_NAMESPACE)
        if instance.path.namespace is None or not instance.path.namespace:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_NAMESPACE)
        cop = self.proxy.pywbem2cmpi_instname(instance.path)
        inst = self.proxy.pywbem2cmpi_inst(instance)
        return self.broker.modifyInstance(self.ctx, cop, inst)
    
    def DeliverIndication(self, ns, instance):
        if self.broker.name() == 'Pegasus':
            allow_null_ns = False
        else:
            allow_null_ns = True
            if self.broker.name() == 'RequestHandler':
                # Check sblim bug #2185410.
                if instance.path is not None:
                    instance.path.namespace = None
        inst = self.proxy.pywbem2cmpi_inst(instance, allow_null_ns)
        rv = self.broker.deliverIndication(self.ctx, ns, inst)
        return rv

    def PrepareAttachThread(self):
        # Return new *BrokerCIMOMHandle*, the context itself would be useless
        new_ctx = self.broker.prepareAttachThread(self.ctx)
        new_broker = BrokerCIMOMHandle(self.proxy, new_ctx)
        return new_broker

    def AttachThread(self):
        return self.broker.attachThread(self.ctx)

    def DetachThread(self):
        return self.broker.detachThread(self.ctx)

    def is_subclass(self, ns, super, sub):
        subObjPath=self.broker.new_object_path(ns, sub)
        return bool(self.broker.classPathIsA(subObjPath,super))

    def bummer(self):
        self.broker.bummer()

_log_pri_map = {
        cmpi.CMPI_SEV_ERROR    :syslog.LOG_ERR,
        cmpi.CMPI_SEV_INFO     :syslog.LOG_INFO,
        cmpi.CMPI_SEV_WARNING  :syslog.LOG_WARNING,
        cmpi.CMPI_DEV_DEBUG    :syslog.LOG_DEBUG,
        }

_trace_prefix_map = {
    cmpi.CMPI_LEV_VERBOSE      :"DEBG",
    cmpi.CMPI_LEV_INFO         :"INFO",
    cmpi.CMPI_LEV_WARNING      :"WARN",
}

class Logger(object):
    def __init__(self, broker, miname):
        #self.broker = ExceptionClassWrapper(broker)
        self.broker = broker
        self.miname = miname
    def __log_message(self, severity, msg):
        try:
            self.broker.LogMessage(severity, self.miname, msg);
        except cmpi.CMPIException, e:
            if e.get_error_code() == cmpi.CMPI_RC_ERR_NOT_SUPPORTED: 
                syslog.syslog(syslog.LOG_DAEMON | _log_pri_map[severity], 
                        '%s: %s' % (self.miname, msg))
    def __trace_message(self, severity, component, msg):
        try:
            self.broker.TraceMessage(severity, component, msg);
        except cmpi.CMPIException, e:
            if e.get_error_code() == cmpi.CMPI_RC_ERR_NOT_SUPPORTED:
                # fall back to log_debug if tracing is not supported
                self.log_debug("%s:%s: %s" % (
                        component, _trace_prefix_map[severity], msg))
    def log_error(self, msg):
        self.__log_message(cmpi.CMPI_SEV_ERROR, msg);
    def log_info(self, msg):
        self.__log_message(cmpi.CMPI_SEV_INFO, msg);
    def log_warn(self, msg):
        self.__log_message(cmpi.CMPI_SEV_WARNING, msg);
    def log_debug(self, msg):
        self.__log_message(cmpi.CMPI_DEV_DEBUG, msg);
    def trace_verbose(self, component, msg):
        self.__trace_message(cmpi.CMPI_LEV_VERBOSE, component, msg)
    def trace_info(self, component, msg):
        self.__trace_message(cmpi.CMPI_LEV_INFO, component, msg)
    def trace_warn(self, component, msg):
        self.__trace_message(cmpi.CMPI_LEV_WARNING, component, msg)

class ProviderEnvironment(object):
    def __init__(self, proxy, ctx):
        self.proxy = proxy
        self.ctx = ContextWrap(proxy, ctx)
    def get_logger(self):
        return Logger(self.proxy.broker, self.proxy.miname)
    def get_cimom_handle(self):
        return BrokerCIMOMHandle(self.proxy, self.ctx.cmpicontext)

g_proxies = {}

def get_cmpi_proxy_provider(miname, broker, ctx):
    try:
        prox = g_proxies[miname]
        if prox.proxy.env.proxy.broker != broker:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, 
                        'New broker not the same as cached broker!')
    except KeyError:
        prox = ExceptionClassWrapper(CMPIProxyProvider(miname, broker, ctx))
        g_proxies[miname] = prox
    return prox


class CMPIProxyProvider(object):

    def __init__(self, miname, broker, ctx):
        self.miname = miname
        self.broker = broker
        env = ProviderEnvironment(self, ctx)
        provmod = miname
        if provmod[0] != '/':
            provmod = '/usr/lib/pycim/' + provmod
        if not provmod.endswith('.py'):
            provmod+= '.py'
        self.proxy = ProviderProxy(env, provmod)
        #print '*** broker.name()', broker.name()
        #print '*** broker.capabilities()', broker.capabilities()
        #print '*** broker.version()', broker.version()

    def enum_instance_names(self, ctx, rslt, objname):
        #test_conversions()
        env = ProviderEnvironment(self, ctx)
        op = self.cmpi2pywbem_instname(objname)
        try:
            for i in self.proxy.MI_enumInstanceNames(env, op):
                cop = self.pywbem2cmpi_instname(i)
                rslt.return_objectpath(cop)
        except pywbem.CIMError, args:
            return args[:2]
        rslt.done()
        return (0, '')

    def enum_instances(self, ctx, rslt, objname, plist):
        env = ProviderEnvironment(self, ctx)
        op = self.cmpi2pywbem_instname(objname)
        try:
            for i in self.proxy.MI_enumInstances(env, op, plist):
                cinst = self.pywbem2cmpi_inst(i)
                rslt.return_instance(cinst)
        except pywbem.CIMError, args:
            return args[:2]
        rslt.done()
        return (0, '')

    def get_instance(self, ctx, rslt, objname, plist):
        env = ProviderEnvironment(self, ctx)
        op = self.cmpi2pywbem_instname(objname)
        try:
            pinst = self.proxy.MI_getInstance(env, op, plist)
        except pywbem.CIMError, args:
            return args[:2]
        cinst = self.pywbem2cmpi_inst(pinst)
        rslt.return_instance(cinst)
        rslt.done()
        return (0, '')


    def create_instance(self, ctx, rslt, objname, newinst):
        env = ProviderEnvironment(self, ctx)
        pinst = self.cmpi2pywbem_inst(newinst)
        try:
            piname = self.proxy.MI_createInstance(env, pinst)
        except pywbem.CIMError, args:
            return args[:2]
        ciname = self.pywbem2cmpi_instname(piname)
        rslt.return_objectpath(ciname)
        rslt.done()
        return (0, '')


    def set_instance(self, ctx, rslt, objname, newinst, plist):
        env = ProviderEnvironment(self, ctx)
        pinst = self.cmpi2pywbem_inst(newinst)
        pinst.path = self.cmpi2pywbem_instname(objname)
        try:
            self.proxy.MI_modifyInstance(env, pinst, plist)
        except pywbem.CIMError, args:
            return args[:2]
        return (0, '')

    def delete_instance(self, ctx, rslt, objname):
        env = ProviderEnvironment(self, ctx)
        piname = self.cmpi2pywbem_instname(objname)
        try:
            self.proxy.MI_deleteInstance(env, piname)
        except pywbem.CIMError, args:
            return args[:2]
        return (0, '')


    def exec_query(self, ctx, rslt, objname, query, lang):
        return (pywbem.CIM_ERR_NOT_SUPPORTED, '')


    def associator_names(self, ctx, rslt, objName, assocClass, resultClass,
            role, resultRole):
        env = ProviderEnvironment(self, ctx)
        piname = self.cmpi2pywbem_instname(objName)

        try:
            for i in self.proxy.MI_associatorNames(env, piname, 
                    assocClass, resultClass, role, resultRole):
                ciname = self.pywbem2cmpi_instname(i)
                rslt.return_objectpath(ciname)
        except pywbem.CIMError, args:
            return args[:2]
        rslt.done()
        return (0, '')

    def associators(self, ctx, rslt, objName, assocClass, resultClass,
            role, resultRole, props):
        env = ProviderEnvironment(self, ctx)
        piname = self.cmpi2pywbem_instname(objName)

        try:
            for i in self.proxy.MI_associators(env, piname, 
                    assocClass, resultClass, role, resultRole, props):
                cinst = self.pywbem2cmpi_inst(i)
                rslt.return_instance(cinst)
        except pywbem.CIMError, args:
            return args[:2]
        rslt.done()
        return (0, '')


    def reference_names(self, ctx, rslt, objName, resultClass, role):
        env = ProviderEnvironment(self, ctx)
        piname = self.cmpi2pywbem_instname(objName)

        try:
            for i in self.proxy.MI_referenceNames(env, piname, 
                    resultClass, role):
                ciname = self.pywbem2cmpi_instname(i)
                rslt.return_objectpath(ciname)
        except pywbem.CIMError, args:
            return args[:2]
        rslt.done()
        return (0, '')


    def references(self, ctx, rslt, objName, resultClass, role, props):
        env = ProviderEnvironment(self, ctx)
        piname = self.cmpi2pywbem_instname(objName)

        try:
            for i in self.proxy.MI_references(env, piname, 
                    resultClass, role, props):
                cinst = self.pywbem2cmpi_inst(i)
                rslt.return_instance(cinst)
        except pywbem.CIMError, args:
            return args[:2]
        rslt.done()
        return (0, '')


    def invoke_method(self, ctx, rslt, objName, method, inargs, outargs):
        env = ProviderEnvironment(self, ctx)
        op = self.cmpi2pywbem_instname(objName)
        pinargs = self.cmpi2pywbem_args(inargs)
        try:
            ((_type, rv), poutargs) = self.proxy.MI_invokeMethod(env, 
                    op, method, pinargs)
        except pywbem.CIMError, args:
            return args[:2]

        self.pywbem2cmpi_args(poutargs, outargs)


        data, _type = self.pywbem2cmpi_value(rv, _type=_type)
        rslt.return_data(data, _pywbem2cmpi_typemap[_type])
        rslt.done()
        return (0, '')


    def authorize_filter(self, ctx, filter, className, classPath, owner):
        env = ProviderEnvironment(self, ctx)
        filt = self.cmpi2pywbem_selectexp(filter)
        classpath = self.cmpi2pywbem_instname(classPath)
        try:
            self.proxy.MI_authorizeFilter(env, 
                    filt, className, classpath, owner)
        except pywbem.CIMError, args:
            #expect an exception if not success
            return args[:2]

        return (0, '')

    def activate_filter(self, ctx, filter, className, classPath, 
            firstActivation):
        env = ProviderEnvironment(self, ctx)
        filt = self.cmpi2pywbem_selectexp(filter)
        classpath = self.cmpi2pywbem_instname(classPath)
        try:
            self.proxy.MI_activateFilter(env, 
                    filt, className, classpath, firstActivation)
        except pywbem.CIMError, args:
            #expect an exception if not success
            return args[:2]

        return (0, '')

    def deactivate_filter(self, ctx, filter, className, classPath, 
            lastActivation):
        env = ProviderEnvironment(self, ctx)
        filt = self.cmpi2pywbem_selectexp(filter)
        classpath = self.cmpi2pywbem_instname(classPath)
        try:
            self.proxy.MI_deActivateFilter(env, 
                    filt, className, classpath, lastActivation)
        except pywbem.CIMError, args:
            #expect an exception if not success
            return args[:2]

        return (0, '')

    #def must_poll(self, ctx, rslt, filter, className, classPath):
    # NOTE: sfcb signature for this doesn't have the rslt. 
    def must_poll(self, ctx, filter, className, classPath):
        # must_poll is not supported by most cimoms, partly because
        # the spec is ambiguous, so commented out for a no-op
        # and just return 1 for FALSE
        '''
        env = ProviderEnvironment(self, ctx)
        filt = self.cmpi2pywbem_selectexp(filter)
        classpath = self.cmpi2pywbem_instname(classPath)
        try:
            rv = self.proxy.MI_mustPoll(env, 
                    filt, className, classpath)
        except pywbem.CIMError, args:
            return args[:2]
        '''
        return (1, '')


    def enable_indications(self, ctx):
        env = ProviderEnvironment(self, ctx)
        try:
            self.proxy.MI_enableIndications(env)
        except pywbem.CIMError, args:
            #expect an exception if not success
            return args[:2]

        return (0, '')

    def disable_indications(self, ctx):
        env = ProviderEnvironment(self, ctx)
        try:
            self.proxy.MI_disableIndications(env)
        except pywbem.CIMError, args:
            #expect an exception if not success
            return args[:2]

        return (0, '')

    def cleanup(self, ctx, terminating):
        env = ProviderEnvironment(self, ctx)
        try:
            if not terminating:    
                if not self.proxy.MI_canunload(env):
                    return (cmpi.CMPI_RC_DO_NOT_UNLOAD, '')
            self.proxy.MI_shutdown(env)
        except pywbem.CIMError, args:
            return args[:2]

        return (0, '')

    # conversion routines
    #######################################################################

    def cmpi2pywbem_inst(self, cmpiinst):
        cop = self.cmpi2pywbem_instname(cmpiinst.objectpath())
        props = {}
        for i in xrange(0, cmpiinst.property_count()):
            data, name = cmpiinst.get_property_at(i)
            _type, is_array = _cmpi_type2string(data.type)
            pval = self.cmpi2pywbem_data(data, _type, is_array)
            prop = pywbem.CIMProperty(name, pval, _type, is_array=is_array)
            props[name] = prop
        inst = pywbem.CIMInstance(cop.classname, props, path=cop)
        return inst

    def cmpi2pywbem_args(self, cargs):
        r = {}
        for i in xrange(0, cargs.arg_count()):
            data, name = cargs.get_arg_at(i)
            _type, is_array = _cmpi_type2string(data.type)
            pval = self.cmpi2pywbem_data(data, _type, is_array)
            r[name] = pval
        return r

    def pywbem2cmpi_args(self, pargs, cargs=None):
        if cargs is None:
            cargs = self.broker.new_args()
        for name, (_type, pval) in pargs.items():
            data, _type = self.pywbem2cmpi_value(pval, _type)
            ctype = _pywbem2cmpi_typemap[_type]
            if isinstance(pval, list):
                ctype = ctype | cmpi.CMPI_ARRAY
            cargs.set(str(name), data, ctype)
        return cargs


    def pywbem2cmpi_inst(self, pinst, allow_null_ns=False):
        pcop = pinst.path
        if not allow_null_ns:
            if pcop is None or pcop.namespace is None:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_NAMESPACE, 
                        "Instance must have a namespace")
        else:
            if pcop is None:
                pcop = pywbem.CIMInstanceName(pinst.classname)
        cop = self.pywbem2cmpi_instname(pcop)
        cinst = self.broker.new_instance(cop, allow_null_ns)
        if pinst.property_list is not None:
            cinst.set_property_filter(pinst.property_list)
        for prop in pinst.properties.values():
        #    if pinst.property_list and \
        #            prop.name.lower() not in pinst.property_list:
        #        continue
            try:
                _type = prop.type
                if _type == "string" and prop.embedded_object:
                    _type = prop.embedded_object
                data, _type = self.pywbem2cmpi_value(prop.value, _type=_type)
            except TypeError, te:
                raise TypeError('Error converting Property %s: Value %s, Type %s; %s' % (prop.name, prop.value, prop.type, str(te)))
            ctype = _pywbem2cmpi_typemap[_type]
            if isinstance(prop.value, list) or prop.is_array:
                ctype = ctype | cmpi.CMPI_ARRAY
            cinst.set_property(str(prop.name), data, ctype)
        return cinst



    def cmpi2pywbem_instname(self, cmpiobjpath):
        keys = {}
        for i in xrange(0, cmpiobjpath.key_count()):
            data,keyname = cmpiobjpath.get_key_at(i)
            pval = self.cmpi2pywbem_data(data)
            keys[keyname] = pval

        rv = pywbem.CIMInstanceName(cmpiobjpath.classname(), 
                keys, namespace=cmpiobjpath.namespace())
        return rv

    def pywbem2cmpi_instname(self, iname):
        cop = self.broker.new_object_path(iname.namespace, str(iname.classname))
        if isinstance(iname, pywbem.CIMInstanceName):
            for name, val in iname.keybindings.items():
                if val is None:
                    raise ValueError('NULL value for key "%s.%s"' % \
                            (iname.classname, name))
                data, _type = self.pywbem2cmpi_value(val)
                cop.add_key(str(name), data, _pywbem2cmpi_typemap[_type])
        return cop
        
    def pywbem2cmpi_value(self, pdata, _type=None, cval=None):
        if pdata is None:
            assert(_type is not None)
            return None, _type
        is_array = isinstance(pdata, list)
        if _type is None:
            if isinstance(pdata, pywbem.CIMInstance):
                _type = 'instance'
            elif isinstance(pdata, pywbem.CIMInstanceName):
                _type = 'reference'
            else:
                _type = pywbem.cimtype(pdata)
        attr = _type
        if cval is None:
            cval = cmpi.CMPIValue()
        if is_array:
            ralen = len(pdata)
            ctype = _pywbem2cmpi_typemap[_type]
            car = self.broker.new_array(ralen, ctype)
            for i, rael in enumerate(pdata):
                cv, tt = self.pywbem2cmpi_value(rael, _type=_type)
                car.set(i, cv, ctype)
            cval.array = car
            return cval, _type
        if _type == 'reference':
            attr = 'ref'
            pdata = self.pywbem2cmpi_instname(pdata)
        elif _type == 'string':
            pdata = self.broker.new_string(str(pdata))
        elif _type == 'datetime':
            attr = 'dateTime'
            pdata = self.pywbem2cmpi_datetime(pdata)
        elif _type == 'instance':
            attr = 'inst'
            pdata = self.pywbem2cmpi_inst(pdata)
        elif _type == 'chars':
            pdata = self.broker.new_string(str(pdata))
        setattr(cval, attr, pdata)
        return cval, _type

    def cmpi2pywbem_value(self, cval, _type, is_array=False):
        ctype = _type
        if _type == 'reference':
            ctype = 'ref' 
        if is_array:
            pval = []
            car = cval.array
            for i in xrange(0, car.size()):
                data = car.at(i)
                ptype = _cmpi2pywbem_typemap[data.type]
                rael = self.cmpi2pywbem_value(data.value, _type)
                pval.append(rael)
        else:
            cval = getattr(cval, ctype)
            if _type == 'string':
                pval = cval.__str__()
            elif _type == 'chars':
                pval = cval.__str__()
            elif ctype == 'ref':
                pval = self.cmpi2pywbem_instname(cval)
            else:
                pval = pywbem.tocimobj(_type, cval)
        return pval



    def pywbem2cmpi_data(self, pdata, _type=None):
        is_array = isinstance(pdata, list)
        if _type is None:
            _type = pywbem.cimtype(pdata)
        # This doesn't work below.  cmpi.CMPIData() takes a CMPIData argument. ??
        data = cmpi.CMPIData()
        data.state = 0
        data.type = _pywbem2cmpi_typemap[_type]
        if is_array:
            data.type = data.type | cmpi.CMPI_ARRAY
        if _type == 'reference':
            _type = 'ref'
            pdata = self.pywbem2cmpi_instname(pdata)
        self.pywbem2cmpi_value(pdata, _type, data.value)
        return data



    def cmpi2pywbem_data(self, cdata, _type=None, is_array=None):
        #TODO check for valid cdata.state
        #TODO error handling
        if (cdata.state & cmpi.CMPI_nullValue) > 0:
            return None
        if _type is None:
            _type, is_array = _cmpi_type2string(cdata.type)
        attr = _type
        if is_array:
            rv = []
            car = cdata.value.array
            if car is None:
                return None
            for i in xrange(0, car.size()):
                adata = car.at(i)
                pdata = self.cmpi2pywbem_data(adata, _type, is_array=False)
                rv.append(pdata)
            return rv
        if cdata.is_null():
            return None
        if attr == 'datetime':
            attr = 'dateTime'
        if attr == 'reference':
            attr = 'ref'
        if attr == 'instance':
            attr = 'inst'
        val = getattr(cdata.value, attr)
        if val is None:
            return None
        if _type == 'string':
            val = str(val)
        if _type == 'boolean':
            val = val == 0 and 'false' or 'true'
        if _type == 'datetime':
            val = self.cmpi2pywbem_datetime(val)
        if _type == 'reference':
            val = self.cmpi2pywbem_instname(val)
        if _type == 'instance':
            val = self.cmpi2pywbem_inst(val)
            return val
        if _type == 'chars':
            _type = 'string'
        return pywbem.tocimobj(_type, val)

    def cmpi2pywbem_datetime(self, dt):
        return pywbem.CIMDateTime(dt.__str__())

    def pywbem2cmpi_datetime(self, dt):
        return self.broker.new_datetime_from_string(str(dt))

    def cmpi2pywbem_selectexp(self, filter):
        return filter.__str__()


_pywbem2cmpi_typemap = {
        'boolean'       : cmpi.CMPI_boolean,
        'real32'        : cmpi.CMPI_real32,
        'real64'        : cmpi.CMPI_real64,
        'uint8'        : cmpi.CMPI_uint8,
        'uint16'        : cmpi.CMPI_uint16,
        'uint32'        : cmpi.CMPI_uint32,
        'uint64'        : cmpi.CMPI_uint64,
        'sint8'        : cmpi.CMPI_sint8,
        'sint16'        : cmpi.CMPI_sint16,
        'sint32'        : cmpi.CMPI_sint32,
        'sint64'        : cmpi.CMPI_sint64,
        'reference'     : cmpi.CMPI_ref,
        'string'        : cmpi.CMPI_string,
        'datetime'      : cmpi.CMPI_dateTime,
        'instance'      : cmpi.CMPI_instance,
        'chars'         : cmpi.CMPI_chars,
        }

_cmpi2pywbem_typemap = {
        0                     : None,
        cmpi.CMPI_boolean     : 'boolean',
        cmpi.CMPI_real32      : 'real32', 
        cmpi.CMPI_real64      : 'real64', 
        cmpi.CMPI_uint8       : 'uint8', 
        cmpi.CMPI_uint16      : 'uint16', 
        cmpi.CMPI_uint32      : 'uint32', 
        cmpi.CMPI_uint64      : 'uint64', 
        cmpi.CMPI_sint8       : 'sint8', 
        cmpi.CMPI_sint16      : 'sint16', 
        cmpi.CMPI_sint32      : 'sint32', 
        cmpi.CMPI_sint64      : 'sint64', 
        cmpi.CMPI_ref         : 'reference', 
        cmpi.CMPI_string      : 'string', 
        cmpi.CMPI_dateTime    : 'datetime', 
        cmpi.CMPI_instance    : 'instance', 
        cmpi.CMPI_chars       : 'chars', 

        #cmpi.CMPI_null        : None,
        #cmpi.CMPI_args        : 'args', 
        #cmpi.CMPI_class       : 'class ', 
        #cmpi.CMPI_filter      : 'filter', 
        #cmpi.CMPI_ptr         : 'ptr', 
        #cmpi.CMPI_charsptr    : 'charsp', 
        #cmpi.CMPI_enumeration : 'enumeration', 
        #cmpi.CMPI_chars       : 'chars ', 
        #cmpi.CMPI_char16      : 'char16'
        }

def _cmpi_type2string(itype):
    """ Convert an unsigned short CMPIType to the string representation of 
    the type.

    returns a two-tuple: (<str_type>, bool_is_array)

    """

    tp = None
    is_array = bool(itype & cmpi.CMPI_ARRAY)

    if is_array:
        itype = itype ^ cmpi.CMPI_ARRAY

    try:
        tp = _cmpi2pywbem_typemap[itype]
    except KeyError:
        raise ValueError('Unknown type: %d' % itype)

    return (tp, is_array)

'''
def traceback2string(_type, value, tb):
    import traceback
    import cStringIO
    iostr = cStringIO.StringIO()
    traceback.print_exception(_type, value, tb, None, iostr)
    s = iostr.getvalue()
    return (s, 'cmpi:' + s.replace('\n', '<br>'))
    '''


def test_conversions(proxy):
    s = 'foo'
    cs, _type = pywbem2cmpi_value(s)
    assert(cs.string.__str__() == s)
    assert(_type == 'string')
    ns = cmpi2pywbem_value(cs, _type)
    assert(s == ns)
    #cdata = cmpi.CMPIData(None)
    i = pywbem.Uint32(5)
    ci, _type = pywbem2cmpi_value(i)
    assert(_type == 'uint32')
    assert(ci.uint32 == i)
    ni = cmpi2pywbem_value(ci, _type)
    assert(isinstance(ni, pywbem.Uint32))
    assert(i == ni)
    l = ['python','is','great']
    cl, _type = pywbem2cmpi_value(l)
    nl = cmpi2pywbem_value(cl, _type, is_array=True)
    assert(nl == l)

    l = [pywbem.Real32(3.4), pywbem.Real32(3.14159)] 
    cl, _type = pywbem2cmpi_value(l)
    nl = cmpi2pywbem_value(cl, _type, is_array=True)
    assert(_type == 'real32')
    for i in xrange(0, len(l)):
        assert(abs(l[i] - nl[i]) <= 0.001)

            
    

    #print s == cmpi2pywbem_

    pcop = pywbem.CIMInstanceName('Cmpi_Swig', namespace='root/cimv2', 
            keybindings={'k1':'A', 'k2':'B'})
    ccop = pywbem2cmpi_instname(pcop)
    ncop = cmpi2pywbem_instname(ccop)
    assert(ncop.classname == 'Cmpi_Swig')
    assert(ncop.namespace == 'root/cimv2')
    assert(len(ncop.keybindings) == 2)
    assert(ncop['k1'] == 'A')
    assert(ncop['k2'] == 'B')

    cinst = cmpi.CMPIInstance(proxy.broker, ccop)
    assert(cinst.objectpath() is not None)

    pinst = pywbem.CIMInstance('Cmpi_Swig', path=pcop, 
            properties={'k1':'A', 'k2':'B',
                'p3':'string prop', 
                'p4':pywbem.Uint32(47)
                })
    cinst = pywbem2cmpi_inst(pinst)
    ninst = cmpi2pywbem_inst(cinst)
    assert(ninst.classname == pinst.classname)
    pinst.properties['n'] = pywbem.CIMProperty('n', None, type='uint32')
    cinst = pywbem2cmpi_inst(pinst)
    ninst = cmpi2pywbem_inst(cinst)
    #assert(ninst['n'] is None)
    assert(ninst.classname == pinst.classname)
    pinst['sint16'] = pywbem.Sint16(16)
    cinst = pywbem2cmpi_inst(pinst)
    ninst = cmpi2pywbem_inst(cinst)
    assert(ninst['sint16'] == 16)
    pinst['sint8'] = pywbem.Sint8(8)
    cinst = pywbem2cmpi_inst(pinst)
    ninst = cmpi2pywbem_inst(cinst)
    assert(ninst['sint8'] == 8)
    pinst['uint8a'] = [pywbem.Uint8(1), pywbem.Uint8(2), pywbem.Uint8(3)]
    cinst = pywbem2cmpi_inst(pinst)
    ninst = cmpi2pywbem_inst(cinst)
    assert(ninst['uint8a'] == pinst['uint8a'])

    pdt = pywbem.CIMDateTime.now()
    cdt = pywbem2cmpi_datetime(pdt)
    ndt = cmpi2pywbem_datetime(cdt)
    assert(pdt == ndt)
    cdt = proxy.broker.new_datetime_from_string('20080623144759.823564-360')
    print '** ctd.is_interval()', cdt.is_interval()
    print '** ctd.__str__()', cdt.__str__()

    pinst['dt'] = pywbem.CIMDateTime('20080623144759.823564-360')
    cinst = pywbem2cmpi_inst(pinst)
    ninst = cmpi2pywbem_inst(cinst)
    print '** ninst["dt"]', ninst['dt']
    print '** pinst["dt"]', pinst['dt']
    assert(ninst['dt'] == pinst['dt'])

    pargs = {
            'one':'one',
            'two':pywbem.Uint32(43), 
            'three':pywbem.CIMDateTime.now(),
            'four':['one','two','three'],
            'five':[pywbem.CIMDateTime.now(), pywbem.CIMDateTime.now()],
            'six':[pywbem.Sint32(-1), pywbem.Sint32(-2)]
            }
    iargs = dict([(k, (pywbem.cimtype(v), v)) for k, v in pargs.items()])
    cargs = pywbem2cmpi_args(iargs)
    nargs = cmpi2pywbem_args(cargs)
    assert(pargs == nargs)


    print '** tests passed' 




#!/usr/bin/env python
import pywbem 
from optparse import OptionParser
from getpass import getpass

def getWBEMConnParserOptions(parser):
    parser.add_option('-u', '--url', default='/tmp/sfcbHttpSocket', help='Specify the URL to the CIMOM')
    parser.add_option('-n', '--namespace', default='root/cimv2', help='Specify the namespace the test runs against')
    parser.add_option('', '--user', default=None, help='Specify the user name used when connection to the CIMOM')
    parser.add_option('', '--password', default=None, help='Specify the password for the user')



def WBEMConnFromOptions(parser=None):
    if parser==None:
        parser = OptionParser()
        getWBEMConnParserOptions(parser)
    options, args = parser.parse_args()
    pw = options.password
    if options.user and not pw:
        pw = getpass('\nEnter password for %s: ' % options.user)
        options.password = pw
    creds = None
    if options.user:
        creds = (options.user, pw)
    wconn = pywbem.WBEMConnection(options.url, creds)
    if options.namespace:
        wconn.default_namespace = options.namespace
    return wconn







#!/usr/bin/env python
#
#Author: Kenny Woodson
#Date: Mon Sep 17, 2007 
#Description: Script to test the functions of the TestAtomProvider.
#Tests include:
# CreateInstance, EnumerateInstance, EnumerateInstanceNames,
#  ModifyInstance, DeleteInstance, GetInstance
#
###############################################################################

import pywbem
from os import path
import subprocess
import unittest
import math
from lib import wbem_connection
from optparse import OptionParser
conn = None

globalParser = None

_tolerance = .04

_atoms = {'Hydrogen': 1,
         'Helium': 2,
         'Lithium': 3,
         'Beryllium': 4,
         'Boron': 5,
         'Carbon': 6,
         'Nitrogen': 7,
         'Oxygen': 8,
         'Fluorine': 9,
         'Neon': 10 }

_atomic_weights = {'Hydrogen': 1.00794,
                   'Helium': 4.002602,
                   'Lithium': 6.941,
                   'Beryllium': 9.012182,
                   'Boron': 10.811,
                   'Carbon': 12.0107,
                   'Nitrogen': 14.0067,
                   'Oxygen': 15.9994,
                   'Fluorine': 18.9984032,
                   'Neon': 20.1797 }

def restart_gmond():
    p = subprocess.Popen([ path.join('/etc/init.d/', "novell-gmond"), \
                          "restart"], stdout=subprocess.PIPE)
    p.wait()


def _compare_values(conn, instance, time):
    types = {'boolProp': 'boolean',
             'dateProp': 'datetime',
             'real32Prop':  'real32',
             'real32Propa': 'real32',
             'real64Prop':  'real64',
             'real64Propa': 'real64',
             'sint16Prop':  'sint16',
             'sint16Propa': 'sint16',
             'sint32Prop':  'sint32',
             'sint32Propa': 'sint32',
             'sint64Prop':  'sint64',
             'sint64Propa': 'sint64',
             'sint8prop':   'sint8',
             'sint8propa':  'sint8',
             'stringProp':  'string',
             'stringPropa': 'string',
             'uint16Prop':  'uint16',
             'uint16Propa': 'uint16',
             'uint32Prop':  'uint32',
             'uint32Propa': 'uint32',
             'uint64Prop':  'uint64',
             'uint64Propa': 'uint64',
             'uint8Prop'  : 'uint8',
             'uint8Propa' : 'uint8' }

    if instance['Name'] in _atoms:
        #print instance['Name']
        atoms_value = _atoms.get(instance['Name'])
        atoms_weight = _atomic_weights[instance['Name']]
        for prop,value in instance.items():
            #print "\nProperty=%s"%str(prop),"Value=%s"%str(value)
            #print "Type=%s"%instance.properties[prop].type
            #Char and Char_array
            if prop == 'char16Prop' or prop == 'char16Propa':
                pass
            #Date Property
            elif prop == 'dateProp':
                if str(instance[prop]) != str(time):
                    raise "DateProp NOT EQUAL"
            #Name or stringProp
            elif prop == 'Name' or prop == 'stringProp':
                if instance['Name'] not in _atoms or \
                   instance['stringProp'] not in _atoms:
                    raise "Atom Name NOT FOUND: %s" & instance['Name']
            #boolProp
            elif prop == 'boolProp':
                if instance[prop] !=  False:
                    raise "False NOT EQUAL False"
            #All values not in lists
            #real64Prop fails this check
            elif (instance.properties[prop].type == types.get(prop)) and \
                  type(instance.properties[prop].value) != type([]):
                if prop == 'uint8Prop':
                    if pywbem.Uint8(atoms_value) != instance[prop]:
                        raise "%s Error: %s" % (prop, instance[prop])
                elif prop.startswith("real"):
                    if _atomic_weights[instance['Name']] != atoms_weight:
                        raise "%s == %s"%(_atomic_weights[instance['Name']],atoms_weight)
                elif atoms_value != value:
                    raise "%s == %s"%(atoms_value,value)
            #All list values
            elif isinstance(instance.properties[prop].value, list) and \
                 instance.properties[prop].type == types.get(prop):
                if prop == 'stringPropa':
                    if value[0] != 'proton' and value[1] != 'electron' \
                       and value[2] != 'neutron':
                        raise "String Array NOT EQUAL"
                elif prop == 'uint8Propa':
                    for val in instance.properties[prop].value:
                        if pywbem.Uint8(atoms_value) != val:
                            raise ("Uint8 Values NOT EQUAL")
                else:
                    #print "\n"
                    #print "instance.properties[prop]=%s"%str(prop)
                    #print "atoms_value=%s"%str(atoms_value)
                    for a_prop in instance.properties[prop].value:
                        #print "a_prop=%s"%str(a_prop)
                        #print "value=%s"%str(value)
                        for num in value: #Array
                            #print "Checking a_prop=%s with num=%s" % (str(a_prop),str(num))
                            #print "startswith=%s"%str(prop.startswith("real"))
                            #if str(instance.properties[prop]).startswith("real"): 
                            if prop.startswith("real"): 
                                #print "Checking num=%s with atoms_weight=%s"%(str(num),str(atoms_weight))
                                if math.fabs(num - atoms_weight) > _tolerance:
                                    raise "%s NOT EQUAL %s" % (str(num), str(atoms_weight))
                            elif a_prop != num:
                                raise "%s NOT EQUAL %s" % (atoms_value, num)
            else:
                raise "%s NOT EQUAL %s" % (atoms_value, value)
    else:
        raise "Instance of Test_Atom not Found: %s" % (instance['Name'])


def get_instance(conn, keybindings, propertylist=None):
    if propertylist is None:
        propertylist = []
    inst = None
    try:
        iname = pywbem.CIMInstanceName(classname='Test_Atom', \
                keybindings=(keybindings), namespace='root/cimv2')
        inst = conn.GetInstance(iname, PropertyList=propertylist)
    except pywbem.CIMError, arg:
        raise (arg)
    return inst


def get_instance_names(conn):
    """
    Open a wbem connection and retrieve the newly created instance names. 
    """

    try:
        ta_list = conn.EnumerateInstanceNames('Test_Atom')
    except pywbem.CIMError, arg:
        raise 
        return None

    return ta_list


def get_test_instances(conn):
    """
    Open a wbem connection and retrieve the newly created instances.
    """

    try:
        ta_list = conn.EnumerateInstances('Test_Atom')
    except pywbem.CIMError, arg:
        raise 
        return None

    return ta_list


def delete_test_instances(conn, del_instances):
    """
    Open a wbem connection and attempt to delete the newly created instance.
    """

    try:
        for atom in del_instances:
            conn.DeleteInstance(atom)
    except pywbem.CIMError, arg:
        raise arg
        return False

    return True

class TestAtomProvider(unittest.TestCase):
    time = pywbem.CIMDateTime.now()

    def setUp(self):
        global conn
        self.inst_paths = []
        self.instance = None
        if conn is None:
            conn = wbem_connection.WBEMConnFromOptions(globalParser)
        unittest.TestCase.setUp(self)

    def tearDown(self):
        for ipath in self.inst_paths:
            try:
                conn.DeleteInstance(ipath)
            except pywbem.CIMError,arg:
                pass
        unittest.TestCase.tearDown(self)



    #def test_1_register(self):
    #    """ Test Register Provider """
    #    testdir = "/usr/lib/pycim"
    #    reginst = pywbem.CIMInstance('OpenWBEM_PyProviderRegistration', \
    #              properties={ 'InstanceID':'TestAtomProvider', \
    #              'NamespaceNames':['root/cimv2'],
    #              'ClassName':'TestAtom',
    #              'ProviderTypes':[pywbem.Uint16(1)], # Indication Handler
    #              'ModulePath':'%s/TestAtomProvider.py' % testdir,
    #               },
    #               path=pywbem.CIMInstanceName('OpenWBEM_PyProviderRegistration',
    #               namespace='Interop')) 
    #        
    #    try:
    #        conn.CreateInstance(reginst)
    #    except pywbem.CIMError, arg:
    #        self.fail("Could not REGISTER %s:%s" % (reginst.classname, str(arg)))
    #    restart_gmond()




    #def test_7_deregister(self):
    #    """ Test Deregister Provider """
    #    conn.default_namespace = 'Interop'
    #    reglist = conn.EnumerateInstanceNames('OpenWBEM_PyProviderRegistration')
    #    for inst_name in reglist:
    #        if inst_name['InstanceID'] == 'TestAtomProvider':
    #            try:
    #                conn.DeleteInstance(inst_name)
    #            except pywbem.CIMError, arg:
    #                self.fail("Could not DEREGISTER Class")

    #    restart_gmond()

    #    try:
    #        conn.GetInstance(inst_name)
    #    except pywbem.CIMError, arg:
    #        self.failUnlessEqual(arg[0], pywbem.CIM_ERR_NOT_FOUND,
    #             'Unexpected exception on GetInstance: %s' % str(arg))


    
    def _create_test_instance(self, name_of_atom, number):
        """ Create a TestAtom instance.  """

        weight = _atomic_weights[name_of_atom]
        #new_instance['char16Prop']  = 
        #new_instance['char16Propa'] = Null
        new_instance = pywbem.CIMInstance('Test_Atom')
        new_instance['Name'] = name_of_atom
        new_instance['boolProp']     = False
        new_instance['dateProp']     = self.time
        new_instance['real32Prop']   = pywbem.Real32(weight)
        new_instance['real32Propa']  = [pywbem.Real32(weight), \
                                        pywbem.Real32(weight), \
                                        pywbem.Real32(weight)]
        new_instance['real64Prop']   = pywbem.Real64(weight)
        new_instance['real64Propa']  = [pywbem.Real64(weight), \
                                        pywbem.Real64(weight), \
                                        pywbem.Real64(weight)]
        new_instance['sint16Prop']   = pywbem.Sint16(number)
        new_instance['sint16Propa']  = [pywbem.Sint16(number), \
                                        pywbem.Sint16(number), \
                                        pywbem.Sint16(number)]
        new_instance['sint32Prop']   = pywbem.Sint32(number)
        new_instance['sint32Propa']  = [pywbem.Sint32(number), \
                                        pywbem.Sint32(number), \
                                        pywbem.Sint32(number)]
        new_instance['sint64Prop']   = pywbem.Sint64(number)
        new_instance['sint64Propa']  = [pywbem.Sint64(number), \
                                        pywbem.Sint64(number), \
                                        pywbem.Sint64(number)]
        new_instance['sint8prop']    = pywbem.Sint8(number)
        new_instance['sint8Propa']   = [pywbem.Sint8(number), \
                                        pywbem.Sint8(number), \
                                        pywbem.Sint8(number)]
        new_instance['stringProp']   = name_of_atom
        new_instance['stringPropa']  = ['proton', 'electron', 'neutron']
        new_instance['uint16Prop']   = pywbem.Uint16(number)
        new_instance['uint16Propa']  = [pywbem.Uint16(number), \
                                        pywbem.Uint16(number), \
                                        pywbem.Uint16(number)]
        new_instance['uint32Prop']   = pywbem.Uint32(number)
        new_instance['uint32Propa']  = [pywbem.Uint32(number), \
                                        pywbem.Uint32(number), \
                                        pywbem.Uint32(number)]
        new_instance['uint64Prop']   = pywbem.Uint64(number)
        new_instance['uint64Propa']  = [pywbem.Uint64(number), \
                                        pywbem.Uint64(number), \
                                        pywbem.Uint64(number)]
        new_instance['uint8Prop']    = pywbem.Uint8(number)
        new_instance['uint8Propa']   = [pywbem.Uint8(number), \
                                        pywbem.Uint8(number), \
                                        pywbem.Uint8(number)]

        try:
            cipath = conn.CreateInstance(new_instance)
            new_instance.path = cipath
            self.inst_paths.append(cipath)

        except pywbem.CIMError, arg:
            return None, arg

        return new_instance, None 


    def test_2_create_instance(self):
        """Test create"""
        for atom_name, atomic_number in _atoms.items():
            rval, msg = self._create_test_instance(atom_name, atomic_number)
            if not rval:
                self.fail('%s: CreateInstance Failed.' % str(msg))
                continue
            try:
                ci = conn.GetInstance(rval.path)
            except pywbem.CIMError,arg:
                self.fail('GetInstance failed for instance just created')
                continue

            _compare_values(conn, ci, self.time)


    def test_3_enum_instances(self):
        """Test enum_instances"""
        insts = []
        for atom_name, atomic_number in _atoms.items():
            rval, msg = self._create_test_instance(atom_name, atomic_number)
            if not rval:
                self.fail('%s: CreateInstance Failed.' % str(msg))
                continue
            try:
                ci = conn.GetInstance(rval.path)
                insts.append(ci)
            except pywbem.CIMError,arg:
                self.fail('GetInstance failed for instance just created')
                continue
        
        try: 
            ta_list = conn.EnumerateInstances('Test_Atom')
        except pywbem.CIMError, arg:
            self.fail('EnumerateInstances Failed: %s' % str(arg))
            raise

        try:
            paths = conn.EnumerateInstanceNames('Test_Atom')
        except pywbem.CIMError, arg:
            self.fail('EnumerateInstanceNames Failed: %s' % str(arg))

        if len(paths) != len(ta_list):
            self.fail('EnumerateInstances returned different number of '
                'results than EnumerateInstanceNames')

        for ci in insts:
            for rci in ta_list:
                rci.path.host = None
                if rci.path == ci.path:
                    _compare_values(conn, rci, self.time)
                    break
            else:
                self.fail('Did not get a created instance back from EnumerateInstance')
                return


    def test_4_enum_instance_names(self):
        """Test enum_instance_names"""
        insts = []
        for atom_name, atomic_number in _atoms.items():
            rval, msg = self._create_test_instance(atom_name, atomic_number)
            if not rval:
                self.fail('%s: CreateInstance Failed.' % str(msg))
                continue
            try:
                ci = conn.GetInstance(rval.path)
                insts.append(ci)
            except pywbem.CIMError,arg:
                self.fail('GetInstance failed for instance just created')
                continue
        
        try: 
            ta_list = conn.EnumerateInstanceNames('Test_Atom')
        except pywbem.CIMError, arg:
            self.fail('EnumerateInstanceNames Failed: %s' % str(arg))
            raise

        try: 
            instances = conn.EnumerateInstances('Test_Atom')
        except pywbem.CIMError, arg:
            self.fail('EnumerateInstances Failed: %s' % str(arg))

        for ci in insts:
            for path in ta_list:
                path.host = None
                if path == ci.path:
                    break
            else:
                self.fail('Did not get a created instance name back from EnumerateNames')
                return


    def test_5_get_instance_with_property_list(self):
        """Test Get_Instance """
        rinst, msg = self._create_test_instance('Carbon', 6)
        if not rinst:
            self.fail('%s: CreateInstance Failed.' % str(msg))
            return

        propertylist = ['uint16Prop', 'dateProp', 'stringProp', 'real64Prop', \
                        'sint32Propa', 'sint32Prop']
        keybindings = {'Name': 'Carbon'}
        try:
            inst = get_instance(conn, keybindings, propertylist)
        except pywbem.CIMError, arg:
            raise arg

        for prop in inst.properties.keys():
            if prop not in inst.path and prop not in propertylist:
                self.fail("Property Not Found in PropertyList: %s" % prop)


    def test_6_modify_instance(self):
        """Test modify instance"""
        rinst, msg = self._create_test_instance('Boron', 5)
        if not rinst:
            self.fail('%s: CreateInstance Failed.' % str(msg))
            return

        propertylist = ['uint64Prop', 'dateProp', 'stringProp', 'real32Prop', 
                        'sint64Propa', 'sint64prop', 'boolProp']
        keybindings = {'Name': 'Boron'}

        mod_instance = get_instance(conn, keybindings, propertylist)

        weight = _atomic_weights['Boron']
        new_time = pywbem.CIMDateTime.now()
        if mod_instance['boolProp']:
            mod_instance['boolProp'] = False
        else:
            mod_instance['boolProp'] = True
        mod_instance['uint64Prop'] = pywbem.Uint64(2)
        mod_instance['dateProp'] = new_time
        mod_instance['stringProp'] = "Helium"
        mod_instance['real32Prop'] = pywbem.Real32(weight)
        mod_instance['sint64Propa'] = pywbem.CIMProperty('sint64Propa', \
                                        value=[pywbem.Sint64(2),  \
                                        pywbem.Sint64(2), pywbem.Sint64(2)])
        mod_instance['sint64prop'] = pywbem.Sint64(2)
        mod_instance['Name'] = 'Boron'

        try:
            conn.ModifyInstance(mod_instance, PropertyList=propertylist)
        except pywbem.CIMError, arg:
            self.fail(arg)

        mod_instance = get_instance(conn, keybindings, propertylist)
        for prop in mod_instance.properties.keys():
            if prop == 'uint64Prop' or prop == 'sint64Prop':
                self.assertEqual(mod_instance[prop],2,"Values NOT EQUAL")
            elif prop == "real32Prop":
                self.assertTrue(math.fabs(mod_instance[prop] - weight) <\
                        _tolerance,"Values NOT EQUAL")
            elif prop == 'dateProp':
                self.assertNotEquals(self.time,mod_instance[prop], \
                                     "Times ARE EQUAL")
            elif prop == 'sint64Propa':
                for val in mod_instance[prop]:
                    self.assertEquals(val, pywbem.Sint64(2), \
                            ("Values NOT EQUAL: " + str(prop)))
            elif prop == 'stringProp':
                self.assertNotEquals(mod_instance[prop], 'Boron', \
                            ("Values NOT EQUAL"+str(prop))) 
            elif prop == 'boolProp':
                self.assertNotEqual(mod_instance['boolProp'], \
                                   (not mod_instance['boolProp']), \
                                    "ModifyInstance failed.") 


    def test_7_delete(self):
        """Test delete instance"""
        insts = []
        for atom_name, atomic_number in _atoms.items():
            rval, msg = self._create_test_instance(atom_name, atomic_number)
            if not rval:
                self.fail('%s: CreateInstance Failed.' % str(msg))
                continue
            try:
                ci = conn.GetInstance(rval.path)
                insts.append(ci)
            except pywbem.CIMError,arg:
                self.fail('GetInstance failed for instance just created')
                continue

        del_instances = get_instance_names(conn)
        for inst in del_instances:
            try:
                conn.DeleteInstance(inst)
            except pywbem.CIMError, arg:
                self.fail('DeleteInstance Failed: %s' % str(arg))
                break
        else:
            for inst in del_instances:
                try:
                    conn.DeleteInstance(inst)
                except pywbem.CIMError, arg:
                    self.failUnlessEqual(arg[0], pywbem.CIM_ERR_NOT_FOUND,
                         'Unexpected exception on delete: %s' % str(arg))

################################################################################
#Return the TestAtomClass
def get_unit_test():
    return TestAtomProvider

if __name__ == "__main__":
    p = OptionParser()
    wbem_connection.getWBEMConnParserOptions(p)
    options, arguments = p.parse_args()
    globalParser = p

    suite = unittest.makeSuite(TestAtomProvider)
    unittest.TextTestRunner(verbosity=2).run(suite)




#!/usr/bin/env python

#import gmetaddata as gmetad
#import gmonddata as gmond
import optparse
import pywbem
from lib import wbem_connection
#import telnetlib as telnet
#import elementtree.ElementTree as ET
#import socket
import unittest
import os
import shutil
import random
import sys

real_tolerance = 0.01
conn = None

#This test requires the usage of elementtree

_g_opts = None
_g_args = None

def _typed_randrange(lo, hi, type):
    if type == 'sint8':
        return pywbem.Sint8(random.randrange(pywbem.Sint8(lo), pywbem.Sint8(hi)))
    elif type == 'sint16':
        return pywbem.Sint16(random.randrange(pywbem.Sint16(lo), pywbem.Sint16(hi)))
    elif type == 'sint32':
        return pywbem.Sint32(random.randrange(pywbem.Sint32(lo), pywbem.Sint32(hi)))
    elif type == 'sint64':
        return pywbem.Sint64(random.randrange(pywbem.Sint64(lo), pywbem.Sint64(hi)))
    elif type == 'uint8':
        return pywbem.Uint8(random.randrange(pywbem.Uint8(lo), pywbem.Uint8(hi)))
    elif type == 'uint16':
        return pywbem.Uint16(random.randrange(pywbem.Uint16(lo), pywbem.Uint16(hi)))
    elif type == 'uint32':
        return pywbem.Uint32(random.randrange(pywbem.Uint32(lo), pywbem.Uint32(hi)))
    elif type == 'uint64':
        return pywbem.Uint64(random.randrange(pywbem.Uint64(lo), pywbem.Uint64(hi)))
    elif type == 'real32':
        return pywbem.Real32(random.randrange(pywbem.Real32(lo), pywbem.Real32(hi)))
    elif type == 'real64':
        return pywbem.Real64(random.randrange(pywbem.Real64(lo), pywbem.Real64(hi)))
        
################################################################################
class TestMethods(unittest.TestCase):

    limits = {'sint8_min':pywbem.Sint8(-128),
            'sint8_max':pywbem.Sint8(127),
            'sint16_min':pywbem.Sint16(-32768),
            'sint16_max':pywbem.Sint16(32767),
            'sint32_min':pywbem.Sint32(-2147483648),
            'sint32_max':pywbem.Sint32(2147483647),
            'sint64_min':pywbem.Sint64(-92233736854775808L),
            'sint64_max':pywbem.Sint64(9223372036854775807L),
            'uint8_min':pywbem.Uint8(0),
            'uint8_max':pywbem.Uint8(0xFF),
            'uint16_min':pywbem.Uint16(0),
            'uint16_max':pywbem.Uint16(0xFFFF),
            'uint32_min':pywbem.Uint32(0),
            'uint32_max':pywbem.Uint32(0xFFFFFFFF),
            'uint64_min':pywbem.Uint64(0L),
            'uint64_max':pywbem.Uint64(0x7FFFFFFFFFFFFFFFL),
            'real32_min':pywbem.Real32(-123456.78),
            'real32_max':pywbem.Real32(123456.78),
            'real64_min':pywbem.Real64(-12345678987654.32),
            'real64_max':pywbem.Real64(12345678987654.32)}
            # note: the last Uint64 value should be 0xFFFFFFFFFFFFFFFF but there is a bug somewhere...
            
    inttypes = ['sint8', 'sint16', 'sint32', 'sint64', 'uint8', 'uint16', 'uint32', 'uint64']
    
    realtypes = ['real32', 'real64']
    
    zeros = {'sint8':pywbem.Sint8(0),
            'sint16':pywbem.Sint16(0),
            'sint32':pywbem.Sint32(0),
            'sint64':pywbem.Sint64(0),
            'uint8':pywbem.Uint8(0),
            'uint16':pywbem.Uint16(0),
            'uint32':pywbem.Uint32(0),
            'uint64':pywbem.Uint64(0),
            'real32':pywbem.Real32(0),
            'real64':pywbem.Real64(0)}
            
    ones = {'sint8':pywbem.Sint8(1),
            'sint16':pywbem.Sint16(1),
            'sint32':pywbem.Sint32(1),
            'sint64':pywbem.Sint64(1),
            'uint8':pywbem.Uint8(1),
            'uint16':pywbem.Uint16(1),
            'uint32':pywbem.Uint32(1),
            'uint64':pywbem.Uint64(1),
            'real32':pywbem.Real32(0),
            'real64':pywbem.Real64(0)}
            
    tens = {'sint8':pywbem.Sint8(10),
            'sint16':pywbem.Sint16(10),
            'sint32':pywbem.Sint32(10),
            'sint64':pywbem.Sint64(10),
            'uint8':pywbem.Uint8(10),
            'uint16':pywbem.Uint16(10),
            'uint32':pywbem.Uint32(10),
            'uint64':pywbem.Uint64(10),
            'real32':pywbem.Real32(10),
            'real64':pywbem.Real64(10)}
            
    twenties = {'sint8':pywbem.Sint8(20),
            'sint16':pywbem.Sint16(20),
            'sint32':pywbem.Sint32(20),
            'sint64':pywbem.Sint64(20),
            'uint8':pywbem.Uint8(20),
            'uint16':pywbem.Uint16(20),
            'uint32':pywbem.Uint32(20),
            'uint64':pywbem.Uint64(20),
            'real32':pywbem.Real32(20),
            'real64':pywbem.Real64(20)}
            
    numlists = {
            'sint8':[pywbem.Sint8(8), pywbem.Sint8(2), pywbem.Sint8(5),
                    pywbem.Sint8(6), pywbem.Sint8(3), pywbem.Sint8(9),
                    pywbem.Sint8(7), pywbem.Sint8(1), pywbem.Sint8(4)],
            'sint16':[pywbem.Sint16(8), pywbem.Sint16(2), pywbem.Sint16(5),
                    pywbem.Sint16(6), pywbem.Sint16(3), pywbem.Sint16(9),
                    pywbem.Sint16(7), pywbem.Sint16(1), pywbem.Sint16(4)],
            'sint32':[pywbem.Sint32(8), pywbem.Sint32(2), pywbem.Sint32(5),
                    pywbem.Sint32(6), pywbem.Sint32(3), pywbem.Sint32(9),
                    pywbem.Sint32(7), pywbem.Sint32(1), pywbem.Sint32(4)],
            'sint64':[pywbem.Sint64(8), pywbem.Sint64(2), pywbem.Sint64(5),
                    pywbem.Sint64(6), pywbem.Sint64(3), pywbem.Sint64(9),
                    pywbem.Sint64(7), pywbem.Sint64(1), pywbem.Sint64(4)],
            'uint8':[pywbem.Uint8(8), pywbem.Uint8(2), pywbem.Uint8(5),
                    pywbem.Uint8(6), pywbem.Uint8(3), pywbem.Uint8(9),
                    pywbem.Uint8(7), pywbem.Uint8(1), pywbem.Uint8(4)],
            'uint16':[pywbem.Uint16(8), pywbem.Uint16(2), pywbem.Uint16(5),
                    pywbem.Uint16(6), pywbem.Uint16(3), pywbem.Uint16(9),
                    pywbem.Uint16(7), pywbem.Uint16(1), pywbem.Uint16(4)],
            'uint32':[pywbem.Uint32(8), pywbem.Uint32(2), pywbem.Uint32(5),
                    pywbem.Uint32(6), pywbem.Uint32(3), pywbem.Uint32(9),
                    pywbem.Uint32(7), pywbem.Uint32(1), pywbem.Uint32(4)],
            'uint64':[pywbem.Uint64(8), pywbem.Uint64(2), pywbem.Uint64(5),
                    pywbem.Uint64(6), pywbem.Uint64(3), pywbem.Uint64(9),
                    pywbem.Uint64(7), pywbem.Uint64(1), pywbem.Uint64(4)],
            'real32':[pywbem.Real32(8), pywbem.Real32(2), pywbem.Real32(5),
                    pywbem.Real32(6), pywbem.Real32(3), pywbem.Real32(9),
                    pywbem.Real32(7), pywbem.Real32(1), pywbem.Real32(4)],
            'real64':[pywbem.Real64(8), pywbem.Real64(2), pywbem.Real64(5),
                    pywbem.Real64(6), pywbem.Real64(3), pywbem.Real64(9),
                    pywbem.Real64(7), pywbem.Real64(1), pywbem.Real64(4)]}

    def _dbgPrint(self, msg=''):
        if self._verbose:
            if len(msg):
                print('\t -- %s --' % msg)
            else:
                print('')

    def setUp(self):
        unittest.TestCase.setUp(self)
        self.conn = conn
        self.conn.debug = True
        for iname in self.conn.EnumerateInstanceNames('Test_Method'):
            self.conn.DeleteInstance(iname)
        self._verbose = _g_opts.verbose
        self._dbgPrint()

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        for iname in self.conn.EnumerateInstanceNames('Test_Method'):
            self.conn.DeleteInstance(iname)

    def _run_and_validate_getrand(self,
            type,
            methodName,
            min,
            max,
            expectedReturnValue=None,
            minReturnValue=None,
            maxReturnValue=None):
        isRealType = False
        if type.startswith('real'):
            isRealType = True
        if isRealType:
            self._dbgPrint('Testing %s invocation with min=%f, max=%f' % (methodName, min, max))
        else:
            self._dbgPrint('Testing %s invocation with min=%d, max=%d' % (methodName, min, max))
        (rv, oparams) = self.conn.InvokeMethod(methodName, 'Test_Method', min=min, max=max)
        if not oparams['success']:
            self.fail('"Success" reported as false for invocation of method %s' % methodName)
        if expectedReturnValue is not None:
            if isRealType:
                self._dbgPrint('Verifying return value (%f) equal to expected return value %f...' % (rv, expectedReturnValue))
                if abs(expectedReturnValue - rv) > real_tolerance:
                    self.fail('Return value not as expected for invocation of method %s' % methodName)
            else:
                self._dbgPrint('Verifying return value (%d) equal to expected return value %d...' % (rv, expectedReturnValue))
                if expectedReturnValue != rv:
                    self.fail('Return value not as expected for invocation of method %s' % methodName)
            self._dbgPrint('Return value is as expected.')
        if minReturnValue is not None:
            if isRealType:
                self._dbgPrint('Verifying return value (%f) >= %f' % (rv, minReturnValue))
            else:
                self._dbgPrint('Verifying return value (%d) >= %d' % (rv, minReturnValue))
            if rv < minReturnValue:
                self.fail('Return value less than expected for invocation of method %s' % methodName)
            self._dbgPrint('Return value is as expected.')
        if maxReturnValue is not None:
            if isRealType:
                self._dbgPrint('Verifying return value (%f) <= %f' % (rv, maxReturnValue))
            else:
                self._dbgPrint('Verifying return value (%d) <= %d' % (rv, maxReturnValue))
            if rv > maxReturnValue:
                self.fail('Return value greater than expected for invocation of method %s' % methodName)
            self._dbgPrint('Return value is as expected.')
            
    def _run_and_validate_getrandlist(self,
            type,
            methodName,
            min,
            max,
            nelems):
        isRealType = False
        if type.startswith('real'):
            isRealType = True
        if isRealType:
            self._dbgPrint('Testing %s invocation with min=%f, max=%f' % (methodName, min, max))
        else:
            self._dbgPrint('Testing %s invocation with min=%d, max=%d' % (methodName, min, max))
        (rv, oparams) = self.conn.InvokeMethod(methodName, 'Test_Method', lo=min, hi=max, nelems=nelems)
        if not rv:
            self.fail('Invocation of %s returned false success value.' % methodName)
        self._dbgPrint('Invocation of %s returned successfully.' % methodName)
        if isRealType:
            self._dbgPrint('Validating lo (%f) and hi (%f) outparams...' % (min, max))
            if abs(oparams['lo'] - min) > real_tolerance:
                self.fail('Returned low range value (%f) not equal to specified value (%f).' % (oparams['lo'], min))
            if abs(oparams['hi'] - max) > real_tolerance:
                self.fail('Returned high range value (%f) not equal to specified value (%f).' % (oparams['hi'], max))
        else:
            self._dbgPrint('Validating lo (%d) and hi (%d) outparams...' % (min, max))
            if oparams['lo'] != min:
                self.fail('Returned low range value (%d) not equal to specified value (%d).' % (oparams['lo'], min))
            if oparams['hi'] != max:
                self.fail('Returned high range value (%d) not equal to specified value (%d).' % (oparams['hi'], max))
        self._dbgPrint('Lo and hi outparams validated successfully.')
        self._dbgPrint('Validating random list values...')
        if oparams['nlist'] is None:
            self.fail('Expected a list of values but got none.')
        if len(oparams['nlist']) != nelems:
            self.fail('Expected a list of %d items but got %d items instead.' % (nelems, len(oparams['nlist'])))
        minkey = '%s_min' % type
        maxkey = '%s_max' % type
        for num in oparams['nlist']:
            if num < TestMethods.limits[minkey] or \
                    num > TestMethods.limits[maxkey]:
                if isRealType:
                    self.fail('List element %f not in expected range for type %s.' % (num, type))
                else:
                    self.fail('List element %d not in expected range for type %s.' % (num, type))
        self._dbgPrint('Random list values validated successfully.')
        
    def _run_and_validate_minmedmax(self, type, methodName, numlist):
        self._dbgPrint('Testing %s invocation' % methodName)
        (rv, oparams) = self.conn.InvokeMethod(methodName, 'Test_Method', numlist=numlist)
        if not rv:
            self.fail('Invocation of %s returned false success value.' % methodName)
        self._dbgPrint('Invocation of %s returned successfully.' % methodName)
        self._dbgPrint('Validating min, median, and max outparams...')
        if oparams['min'] != 1:
            self.fail('Expected min of 1 but instead got %d' % oparams['min'])
        if oparams['max'] != 9:
            self.fail('Expected max of 9 but instead got %d' % oparams['max'])
        if oparams['med'] != 5:
            self.fail('Expected median of 5 but instead got %d' % oparams['med'])
        self._dbgPrint('Min, median, and max values validated successfully.')
            
    def _run_numeric_type_tests(self, typelist):
        gr = self._run_and_validate_getrand
        grl = self._run_and_validate_getrandlist
        mmm = self._run_and_validate_minmedmax
        for type in typelist:
            method = 'genRand_%s' % type
            minkey = '%s_min' % type
            maxkey = '%s_max' % type
            min = TestMethods.limits[minkey]
            max = TestMethods.limits[maxkey]
            gr(type, method, min, max, None, min, max)
            gr(type, method, min, min, min)
            gr(type, method, max, max, max)
            if min != 0:
                gr(type, method, TestMethods.zeros[type], TestMethods.zeros[type], TestMethods.zeros[type])
            gr(type, method, TestMethods.tens[type], TestMethods.twenties[type], None, TestMethods.tens[type], TestMethods.twenties[type])
            # the next two should cause exceptions; getting a TypeError exception is not an error in this case.
            try:
                gr(type, method, min-1, min-1, min-1)
            except TypeError:
                pass
            try:
                gr(type, method, max+1, max+1, max+1)
            except TypeError:
                pass
                
            method = 'genRandList_%s' % type
            nelems = _typed_randrange(TestMethods.tens[type], TestMethods.twenties[type], type)
            grl(type, method, min, max, nelems)
            grl(type, method, min, max, TestMethods.ones[type])
            grl(type, method, min, max, TestMethods.zeros[type])
            if min != 0:
                grl(type, method, TestMethods.zeros[type], max, nelems)
            else:
                grl(type, method, min, TestMethods.zeros[type], nelems)
            grl(type, method, TestMethods.tens[type], TestMethods.twenties[type], nelems)
            grl(type, method, TestMethods.tens[type], TestMethods.twenties[type], TestMethods.ones[type])
            grl(type, method, TestMethods.tens[type], TestMethods.twenties[type], TestMethods.zeros[type])
            
            method = 'minmedmax_%s' % type
            mmm(type, method, TestMethods.numlists[type])
    
        
    def test_integer_types(self):
        self._run_numeric_type_tests(TestMethods.inttypes)
            
    def test_real_types(self):
        self._run_numeric_type_tests(TestMethods.realtypes)
        
    def test_refs(self):
        inst = pywbem.CIMInstance('Test_Method', properties={
                'id':'one', 
                'p_str':'One',
                'p_sint32':pywbem.Sint32(1)})
        self.conn.CreateInstance(inst)

        iname = pywbem.CIMInstanceName('Test_Method', namespace='root/cimv2',
                keybindings={'id':'one'})
        rv, outs = self.conn.InvokeMethod('getStrProp', iname)
        self.assertEquals(rv, 'One')

        rv, outs = self.conn.InvokeMethod('setStrProp', iname, value='won')
        self.assertFalse(outs)
        self.assertEquals(rv, 'One')
        rv, outs = self.conn.InvokeMethod('getStrProp', iname)
        self.assertEquals(rv, 'won')
        inst = self.conn.GetInstance(iname)
        self.assertEquals(inst['p_str'], 'won')

        rv, outs = self.conn.InvokeMethod('getIntProp', iname)
        self.assertEquals(rv, 1)
        self.assertTrue(isinstance(rv, pywbem.Sint32))
        self.assertEquals(inst['p_sint32'], 1)
        rv, outs = self.conn.InvokeMethod('setIntProp', iname, 
                value=pywbem.Sint32(2))
        self.assertTrue(isinstance(rv, pywbem.Sint32))
        self.assertEquals(rv, 1)
        self.assertFalse(outs)
        rv, outs = self.conn.InvokeMethod('getIntProp', iname)
        self.assertEquals(rv, 2)
        self.assertTrue(isinstance(rv, pywbem.Sint32))
        inst = self.conn.GetInstance(iname)
        self.assertEquals(inst['p_sint32'], 2)

        rv, outs = self.conn.InvokeMethod('getObjectPath', 'Test_Method')
        self.assertTrue(isinstance(outs['path'], pywbem.CIMInstanceName))
        self.assertEquals(outs['path']['id'], 'one')

        inst = pywbem.CIMInstance('Test_Method', properties={
                'id':'two', 
                'p_str':'Two',
                'p_sint32':pywbem.Sint32(2)})
        self.conn.CreateInstance(inst)

        rv, outs = self.conn.InvokeMethod('getObjectPaths', 'Test_Method')
        self.assertEquals(len(outs['paths']), 2)
        self.assertTrue(isinstance(outs['paths'][0], pywbem.CIMInstanceName))
        to_delete = outs['paths']

        inst = pywbem.CIMInstance('Test_Method', properties={
                'id':'three', 
                'p_str':'Three',
                'p_sint32':pywbem.Sint32(3)})
        self.conn.CreateInstance(inst)

        iname = pywbem.CIMInstanceName('Test_Method', namespace='root/cimv2', 
                keybindings={'id':'three'})

        inames = self.conn.EnumerateInstanceNames('Test_Method')
        self.assertEquals(len(inames), 3)
        rv, outs = self.conn.InvokeMethod('delObject', 'Test_Method', 
                path=iname)

        inames = self.conn.EnumerateInstanceNames('Test_Method')
        self.assertEquals(len(inames), 2)

        self.conn.CreateInstance(inst)

        '''  # OpenWBEM is broken!  uncomment this for Pegasus.  '''
        rv, outs = self.conn.InvokeMethod('delObjects', 'Test_Method', 
                paths=to_delete)
        
        inames = self.conn.EnumerateInstanceNames('Test_Method')
        self.assertEquals(len(inames), 1)
        self.assertEquals(inames[0]['id'], 'three')

    def test_mkUniStr_sint8(self):
        s = 'Scrum Rocks!'
        l = [pywbem.Sint8(ord(x)) for x in s]
        rv, outs = self.conn.InvokeMethod('mkUniStr_sint8', 'Test_Method', 
                cArr=l)
        self.assertFalse(outs)
        self.assertEquals(rv, s)
        rv, outs = self.conn.InvokeMethod('mkUniStr_sint8', 'Test_Method', 
                cArr=[])
        self.assertEquals(rv, '')

    def test_strCat(self):
        ra = ['one','two','three','four']
        rv, outs = self.conn.InvokeMethod('strCat', 'Test_Method', 
                strs=ra, sep=',')
        self.assertEquals(rv, ','.join(ra))
        self.assertFalse(outs)

    def test_strSplit(self):
        ra = 'one,two,three,four'
        rv, outs = self.conn.InvokeMethod('strSplit', 'Test_Method', 
                str=ra, sep=',')
        self.assertEquals(outs['nelems'], 4)
        self.assertEquals(outs['elems'], ra.split(','))
        self.assertTrue(ra)

    def test_getDate(self):
        dt = pywbem.CIMDateTime.now()
        rv, outs = self.conn.InvokeMethod('getDate', 'Test_Method', 
                datestr=str(dt))
        self.assertFalse(outs)
        self.assertEquals(rv, dt)
        self.assertEquals(str(rv), str(dt))
        self.assertTrue(isinstance(rv, pywbem.CIMDateTime))

    def test_getDates(self):
        dt = pywbem.CIMDateTime.now()
        s1 = str(dt)
        ra = [s1]
        dt = pywbem.CIMDateTime(pywbem.datetime.now() + \
                pywbem.timedelta(seconds=10))
        s2 = str(dt)
        ra.append(s2)
        dt = pywbem.CIMDateTime(pywbem.datetime.now() + \
                pywbem.timedelta(seconds=10))
        s3 = str(dt)
        ra.append(s3)

        rv, outs = self.conn.InvokeMethod('getDates', 'Test_Method', 
                datestrs=ra)
        self.assertTrue(rv)
        self.assertTrue(isinstance(rv, bool))
        self.assertEquals(outs['nelems'], len(ra))
        self.assertTrue(isinstance(outs['nelems'], pywbem.Sint32))

        for i in range(0, len(ra)):
            self.assertTrue(isinstance(outs['elems'][i], pywbem.CIMDateTime))
            self.assertEquals(str(outs['elems'][i]), ra[i])

    def test_minmedmax(self):
        for tstr in ['Sint8', 'Uint8', 'Sint16', 'Uint16', 'Sint32', 'Uint32',
                  'Sint64', 'Uint64', 'Real32', 'Real64']:
            dt = getattr(pywbem, tstr)
            method = 'minmedmax_%s' % tstr
            numlist = [
                dt(2),
                dt(5),
                dt(8),
                dt(1),
                dt(9),
                dt(6),
                dt(4),
                dt(7),
                dt(3),
                ]
            rv, outs = self.conn.InvokeMethod(method, 'Test_Method', 
                    numlist=numlist)
            self.assertTrue(rv)
            self.assertTrue(isinstance(rv, bool))
            self.assertTrue(isinstance(outs['min'], dt))
            self.assertTrue(isinstance(outs['med'], dt))
            self.assertTrue(isinstance(outs['max'], dt))
            self.assertEquals(outs['min'], 1)
            self.assertEquals(outs['max'], 9)
            self.assertEquals(outs['med'], 5)


    def test_xembeddedinst_input_params(self):
        iname = pywbem.CIMInstanceName('Test_Method', namespace='root/cimv2',
                keybindings = {'id':'one'})
        inst = pywbem.CIMInstance('Test_Method', path=None, 
                properties={'p_str':'str1', 'p_sint32':pywbem.Sint32(1)})
        inst.update(iname)
        rv, outs = self.conn.InvokeMethod('createObject', 'Test_Method', 
                    inst=inst)
        insts = self.conn.EnumerateInstances('Test_Method')
        self.assertEquals(len(insts), 1)
        ninst = self.conn.GetInstance(iname)
        self.assertEquals(ninst['p_str'], 'str1')
        self.assertEquals(ninst['p_sint32'], 1)

        iname2 = pywbem.CIMInstanceName('Test_Method', namespace='root/cimv2',
                keybindings = {'id':'two'})
        inst2 = pywbem.CIMInstance('Test_Method', path=None, 
                properties={'p_str':'str2', 'p_sint32':pywbem.Sint32(2)})
        inst2.update(iname2)


        self.conn.DeleteInstance(iname)
        rv, outs = self.conn.InvokeMethod('createObjects', 'Test_Method', 
                    insts=[inst, inst2])
        insts = self.conn.EnumerateInstances('Test_Method')
        self.assertEquals(len(insts), 2)
        ninst = self.conn.GetInstance(iname2)
        self.assertEquals(ninst['p_str'], 'str2')
        self.assertEquals(ninst['p_sint32'], 2)


    def test_xembeddedinst_output_params(self):
        rv,outs = self.conn.InvokeMethod('getBrokerName', 'Test_Method')
        if rv == 'RequestHandler':
            print '\n*** Broker detected as SFCB.  Skipping embedded out param test.'
            print 'Re-enable when SFCB is fixed.' 
            return
        iname = pywbem.CIMInstanceName('Test_Method', namespace='root/cimv2',
                keybindings = {'id':'one'})
        inst = pywbem.CIMInstance('Test_Method', path=None, 
                properties={'p_str':'str1', 'p_sint32':pywbem.Sint32(1)})
        inst.update(iname)

        iname2 = pywbem.CIMInstanceName('Test_Method', namespace='root/cimv2',
                keybindings = {'id':'two'})
        inst2 = pywbem.CIMInstance('Test_Method', path=None, 
                properties={'p_str':'str2', 'p_sint32':pywbem.Sint32(2)})
        inst2.update(iname2)


        rv, outs = self.conn.InvokeMethod('createObjects', 'Test_Method', 
                    insts=[inst, inst2])

        rv, outs = self.conn.InvokeMethod('getObjects', 'Test_Method')
        insts = outs['insts']
        self.assertEquals(len(insts), 2)
        insts.sort()

        self.assertEquals(insts[0]['id'], 'one')
        self.assertEquals(insts[0]['p_str'], 'str1')
        self.assertEquals(insts[0]['p_sint32'], 1)

        self.assertEquals(insts[1]['id'], 'two')
        self.assertEquals(insts[1]['p_str'], 'str2')
        self.assertEquals(insts[1]['p_sint32'], 2)

def get_unit_test():
    return TestMethods


if __name__ == '__main__':
    parser = optparse.OptionParser()
    wbem_connection.getWBEMConnParserOptions(parser)
    parser.add_option('--verbose', '', action='store_true', default=False,
            help='Show verbose output')
    parser.add_option('--level',
            '-l',
            action='store',
            type='int',
            dest='dbglevel',
            help='Indicate the level of debugging statements to display (default=2)',
            default=2)
    _g_opts, _g_args = parser.parse_args()
    conn = wbem_connection.WBEMConnFromOptions(parser)
    
    suite = unittest.makeSuite(TestMethods)
    unittest.TextTestRunner(verbosity=_g_opts.dbglevel).run(suite)




#!/usr/bin/python
#
# Simple indication receiver using Twisted Python.  HTTP post requests
# are listened for on port 5988 and port 5899 using SSL.
#
# Requires Twisted Python and 
#

import sys
import optparse
from twisted.internet import reactor, ssl
from twisted.web import server, resource
import pywbem
import threading
from socket import getfqdn

from twisted.python import log

import time 
from lib import wbem_connection
_interop_ns = None

_port = 5309
_num_to_send = pywbem.Uint16(42)

class CIMListener(resource.Resource):
    """ CIM Listener
    """

    isLeaf = 1


    def __init__(self, callback, http_port=5988): 
        self.callback = callback
        self.http_port = http_port

        site = server.Site(self)

        reactor.listenTCP(self.http_port, site)

    def stop(self):
        reactor.stop()

    def run(self):
        reactor.run()
        
    def render_POST(self, request):
        tt = pywbem.parse_cim(pywbem.xml_to_tupletree(request.content.read()))
        insts = [x[1] for x in tt[2][2][0][2][2]]
        for inst in insts:
            self.callback(inst)
        return ''

def createFilter( ch, query='select * from CIM_ProcessIndication',
                  ns=_interop_ns,
                  querylang='WQL',
                  src_ns='root/cimv2',
                  in_name=None):
    name = in_name or 'cimfilter%s'%time.time()
    filterinst=pywbem.CIMInstance('CIM_IndicationFilter')
    filterinst['CreationClassName']='CIM_IndicationFilter'
    filterinst['SystemCreationClassName']='CIM_ComputerSystem'
    filterinst['SystemName']=getfqdn()
    filterinst['Name']=name
    filterinst['Query']=query
    filterinst['QueryLanguage']=querylang
    filterinst['SourceNamespace']=src_ns
    cop = pywbem.CIMInstanceName('CIM_IndicationFilter')
    cop.keybindings = { 'CreationClassName':'CIM_IndicationFilter',
                        'SystemClassName':'CIM_ComputerSystem',
                        'SystemName':getfqdn(),
                        'Name':name }
    cop.namespace=ns
    filterinst.path = cop
    filtercop = ch.CreateInstance(filterinst)
    return filtercop

def deleteAllSubs(ch, destination='http://localhost:%s' % _port,
                ns=_interop_ns):
    subs = ch.EnumerateInstanceNames('CIM_IndicationSubscription', 
            namespace=ns)
    num = 0
    for sub in subs:
        handler_name = sub['Handler']
        try:
            handler = ch.GetInstance(handler_name, PropertyList=['Destination'])
        except pywbem.CIMError, args:
            print "** Error fetching handler instance: %s %s" % \
                    (handler_name, args)
            continue
        if handler['Destination'] == destination:
            deleteSubscription(ch, sub)
            num+= 1
    if num > 0:
        print '** deleted %d subscriptions' % num


def createDest( ch, destination='http://localhost:%s' % _port,
                ns=_interop_ns,
                in_name=None):
    name = in_name or 'cimlistener%s'%time.time()
    destinst=pywbem.CIMInstance('CIM_ListenerDestinationCIMXML')
    destinst['CreationClassName']='CIM_ListenerDestinationCIMXML'
    destinst['SystemCreationClassName']='CIM_ComputerSystem'
    destinst['SystemName']=getfqdn()
    destinst['Name']=name
    destinst['Destination']=destination
    cop = pywbem.CIMInstanceName('CIM_ListenerDestinationCIMXML')
    cop.keybindings = { 'CreationClassName':'CIM_ListenerDestinationCIMXML',
                        'SystemClassName':'CIM_ComputerSystem',
                        'SystemName':getfqdn(),
                        'Name':name }
    cop.namespace=ns
    destinst.path = cop
    destcop = ch.CreateInstance(destinst)
    return destcop

def createSubscription(ch, ns=_interop_ns):
    replace_ns = ch.default_namespace
    ch.default_namespace=ns
    indfilter=createFilter(ch, ns=ns)
    indhandler=createDest(ch, ns=ns)
    subinst=pywbem.CIMInstance('CIM_IndicationSubscription')
    subinst['Filter']=indfilter
    subinst['Handler']=indhandler
    cop = pywbem.CIMInstanceName('CIM_IndicationSubscription')
    cop.keybindings = { 'Filter':indfilter,
                        'Handler':indhandler }
    cop.namespace=ns
    subinst.path = cop
    subcop = ch.CreateInstance(subinst)
    ch.default_namespace=replace_ns
    return subcop


def deleteSubscription(ch, subcop):
    indfilter = subcop['Filter']
    indhandler= subcop['Handler']
    ch.DeleteInstance(subcop)
    ch.DeleteInstance(indfilter)
    ch.DeleteInstance(indhandler)

# end indication support methods

#log.startLogging(sys.stdout)

_lock = threading.RLock()
_shutdown = False
_insts_received = 0

if __name__ == '__main__':
    parser = optparse.OptionParser()
    wbem_connection.getWBEMConnParserOptions(parser)
    parser.add_option('--verbose', '', action='store_true', default=False,
            help='Show verbose output')
    parser.add_option('--level',
            '-l',
            action='store',
            type='int',
            dest='dbglevel',
            help='Indicate the level of debugging statements to display (default=2)',
            default=2)
    _g_opts, _g_args = parser.parse_args()
    conn = wbem_connection.WBEMConnFromOptions(parser)

    nss = ['root/interop', 'root/cimv2']
    for ns in nss:
        try:
            conn.GetClass('CIM_IndicationSubscription', namespace=ns)
            _interop_ns = ns
            break
        except pywbem.CIMError, arg:
            if arg[0] in [pywbem.CIM_ERR_INVALID_CLASS,
                    pywbem.CIM_ERR_INVALID_NAMESPACE]:
                continue
            raise
    else:
        print 'Unable to find class CIM_IndicationSubscription in namespaces',\
                nss
        sys.exit(1)
    
    def cb(inst):
        global _lock
        global _shutdown
        global _insts_received
        global _num_to_send
        sys.stdout.write('.'); sys.stdout.flush()
        _lock.acquire()
        _insts_received+= 1
        if _num_to_send == _insts_received:
            _shutdown = True
        _lock.release()

    cl = CIMListener(callback=cb, http_port=5309)

    deleteAllSubs(conn, ns=_interop_ns)

    def threadfunc():
        try:
            time.sleep(1)
            numrcv = 0
            subcop = createSubscription(conn, ns=_interop_ns)
            time.sleep(1)
            conn.InvokeMethod('reset_indication_count', 'Test_UpcallAtom')
            print 'Waiting for %s indications...' % _num_to_send
            countsent,outs = conn.InvokeMethod('send_indications', 
                    'Test_UpcallAtom', num_to_send=_num_to_send)
            numsent,outs = conn.InvokeMethod('get_indication_send_count', 
                    'Test_UpcallAtom')
            deleteSubscription(conn, subcop)
            if (countsent != numsent):
                print("\nsend_indications NumSent(%d) doesn't match get_indication_send_count NumSent(%d)\n"%(countsent, numsent));
                sys.exit(1)
            for i in xrange(20):
                _lock.acquire()
                if _shutdown:
                    reactor.stop()
                _lock.release()
                if not reactor.running:
                    break
                time.sleep(.5)
            if reactor.running:
                reactor.stop()
        except:
            if reactor.running:
                reactor.stop()
            raise

    thread = threading.Thread(target=threadfunc)
    thread.start()
    reactor.run()
    print ''
    if _num_to_send != _insts_received:
        print 'Expected %s exceptions, got %s' % (_num_to_send, _insts_received)
        print 'Indication Tests failed!' 
        sys.exit(1)
    else:
        print 'Indication Tests passed.' 




"""Python Provider for Py_UnixProcess

Instruments the CIM class Py_UnixProcess

"""

import pywbem
from cim_provider import CIMProvider

import os
from socket import getfqdn

class Py_UnixProcessProvider(CIMProvider):
    """Instrument the CIM class Py_UnixProcess 

    Model a Linux Process, For use with PyWBEM Provider QuickStart Guide
    
    """

    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        # If you will be filtering instances yourself according to 
        # property_list, role, result_role, and result_class_name 
        # parameters, set self.filter_results to False
        # self.filter_results = False

    def get_instance(self, env, model, property_list):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The 
            key properties are set on this instance to correspond to the 
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the 
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the 
            values, and the instance will be filtered for you. 
        property_list -- The PropertyList from the request. 

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
        

        # TODO fetch system resource matching the following keys:
        #   model['CreationClassName']
        #   model['OSCreationClassName']
        #   model['Handle']
        #   model['CSCreationClassName']
        #   model['OSName']
        #   model['CSName']
        #

        pid = model['handle']
        if not os.path.exists('/proc/' + pid):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

        fo = open('/proc/'+pid+'/stat', 'r')
        st = fo.read()
        fo.close()
        head, sep, tail = st.partition('(')
        name, sep, tail = tail.partition(')')
        model['Name'] = name

        #model['AvailableRequestedStates'] = [self.Values.AvailableRequestedStates.<VAL>,] # TODO 
        #model['Caption'] = '' # TODO 
        #model['CommunicationStatus'] = self.Values.CommunicationStatus.<VAL> # TODO 
        #model['CreationDate'] = pywbem.CIMDateTime() # TODO 
        #model['Description'] = '' # TODO 
        #model['DetailedStatus'] = self.Values.DetailedStatus.<VAL> # TODO 
        #model['ElementName'] = '' # TODO 
        #model['EnabledDefault'] = self.Values.EnabledDefault.Enabled # TODO 
        #model['EnabledState'] = self.Values.EnabledState.Not_Applicable # TODO 
        #model['ExecutionState'] = self.Values.ExecutionState.<VAL> # TODO 
        #model['Generation'] = pywbem.Uint64() # TODO 
        #model['HealthState'] = self.Values.HealthState.<VAL> # TODO 
        #model['InstallDate'] = pywbem.CIMDateTime() # TODO 
        #model['InstanceID'] = '' # TODO 
        #model['KernelModeTime'] = pywbem.Uint64() # TODO 
        #model['ModulePath'] = '' # TODO 
        #model['OperatingStatus'] = self.Values.OperatingStatus.<VAL> # TODO 
        #model['OperationalStatus'] = [self.Values.OperationalStatus.<VAL>,] # TODO 
        #model['OtherEnabledState'] = '' # TODO 
        #model['OtherExecutionDescription'] = '' # TODO 
        #model['Parameters'] = ['',] # TODO 
        #model['ParentProcessID'] = '' # TODO (Required)
        #model['PrimaryStatus'] = self.Values.PrimaryStatus.<VAL> # TODO 
        #model['Priority'] = pywbem.Uint32(0) # TODO 
        #model['ProcessGroupID'] = pywbem.Uint64() # TODO (Required)
        #model['ProcessNiceValue'] = pywbem.Uint32() # TODO 
        #model['ProcessSessionID'] = pywbem.Uint64() # TODO 
        #model['ProcessTTY'] = '' # TODO 
        #model['ProcessWaitingForEvent'] = '' # TODO 
        #model['RealUserID'] = pywbem.Uint64() # TODO (Required)
        #model['RequestedState'] = self.Values.RequestedState.Not_Applicable # TODO 
        #model['Status'] = self.Values.Status.<VAL> # TODO 
        #model['StatusDescriptions'] = ['',] # TODO 
        #model['TerminationDate'] = pywbem.CIMDateTime() # TODO 
        #model['TimeOfLastStateChange'] = pywbem.CIMDateTime() # TODO 
        #model['TransitioningToState'] = self.Values.TransitioningToState.Not_Applicable # TODO 
        #model['UnixPriority'] = pywbem.Sint32() # TODO 
        #model['UserModeTime'] = pywbem.Uint64() # TODO 
        #model['WorkingSetSize'] = pywbem.Uint64() # TODO 
        return model

    def enum_instances(self, env, model, property_list, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method. 
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.  
            The properties of the model are already filtered according to 
            the PropertyList from the request.  Only properties present in 
            the model need to be given values.  If you prefer, you can 
            always set all of the values, and the instance will be filtered 
            for you. 
        property_list -- The PropertyList from the request. 
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
                
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model. 
        model.path.update({u'Handle': None, u'CSName': None, u'OSName': None, u'CSCreationClassName': None, u'OSCreationClassName': None, u'CreationClassName': None})
        
        model['CreationClassName'] = 'Py_UnixProcess'    
        model['OSCreationClassName'] = 'CIM_OperatingSystem'
        model['CSCreationClassName'] = 'CIM_ComputerSystem'
        model['OSName'] = 'Linux'
        model['CSName'] = getfqdn()
        for file in os.listdir('/proc'):
            if not file.isdigit():
                continue
            model['Handle'] = file
            if keys_only:
                yield model
            else:
                try:
                    yield self.get_instance(env, model, property_list)
                except pywbem.CIMError, (num, msg):
                    if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                                   pywbem.CIM_ERR_ACCESS_DENIED):
                        raise

    def set_instance(self, env, instance, modify_existing, property_list):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing 
            instance, the properties on this instance have been filtered by 
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance
        property_list -- The PropertyList from the request. 

        Return the new instance.  The keys must be set on the new instance. 

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only 
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid 
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)
        # TODO create or modify the instance
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        return instance

    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance 
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified 
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """ 

        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)

        # TODO delete the resource
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED) # Remove to implement
        
    def cim_method_requeststatechange(self, env, object_name,
                                      param_requestedstate,
                                      param_timeoutperiod):
        """Implements Py_UnixProcess.RequestStateChange()

        Requests that the state of the element be changed to the value
        specified in the RequestedState parameter. When the requested
        state change takes place, the EnabledState and RequestedState of
        the element will be the same. Invoking the RequestStateChange
        method multiple times could result in earlier requests being
        overwritten or lost. \nA return code of 0 shall indicate the state
        change was successfully initiated. \nA return code of 3 shall
        indicate that the state transition cannot complete within the
        interval specified by the TimeoutPeriod parameter. \nA return code
        of 4096 (0x1000) shall indicate the state change was successfully
        initiated, a ConcreteJob has been created, and its reference
        returned in the output parameter Job. Any other return code
        indicates an error condition.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method RequestStateChange() 
            should be invoked.
        param_requestedstate --  The input parameter RequestedState (type pywbem.Uint16 self.Values.RequestStateChange.RequestedState) 
            The state requested for the element. This information will be
            placed into the RequestedState property of the instance if the
            return code of the RequestStateChange method is 0 (\'Completed
            with No Error\'), or 4096 (0x1000) (\'Job Started\'). Refer to
            the description of the EnabledState and RequestedState
            properties for the detailed explanations of the RequestedState
            values.
            
        param_timeoutperiod --  The input parameter TimeoutPeriod (type pywbem.CIMDateTime) 
            A timeout period that specifies the maximum amount of time that
            the client expects the transition to the new state to take.
            The interval format must be used to specify the TimeoutPeriod.
            A value of 0 or a null parameter indicates that the client has
            no time requirements for the transition. \nIf this property
            does not contain 0 or null and the implementation does not
            support this parameter, a return code of \'Use Of Timeout
            Parameter Not Supported\' shall be returned.
            

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.RequestStateChange)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...)) 
            May contain a reference to the ConcreteJob created to track the
            state transition initiated by the method invocation.
            

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, 
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not 
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor 
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_requeststatechange()' \
                % self.__class__.__name__)

        # TODO do something
        raise pywbem.CIMError(pywbem.CIM_ERR_METHOD_NOT_AVAILABLE) # Remove to implemented
        out_params = []
        #out_params+= [pywbem.CIMParameter('job', type='reference', 
        #                   value=pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))] # TODO
        #rval = # TODO (type pywbem.Uint32 self.Values.RequestStateChange)
        return (rval, out_params)
        
    def cim_method_kill(self, env, object_name,
                        param_signal):
        """Implements Py_UnixProcess.kill()

        Send a signal to a process.
        
        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName 
            specifying the object on which the method kill() 
            should be invoked.
        param_signal --  The input parameter signal (type pywbem.Uint16 self.Values.kill.signal) 
            The signal to send the process
            

        Returns a two-tuple containing the return value (type pywbem.Sint32)
        and a list of CIMParameter objects representing the output parameters

        Output parameters: none

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, 
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not 
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor 
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.cim_method_kill()' \
                % self.__class__.__name__)

        pid = object_name['handle']
        if not os.path.isdir('/proc/' + pid):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                         'Process %s does not exist' % pid)
        rval = pywbem.Sint32(0)
        try:
            os.kill(int(pid), param_signal)
        except OSError, arg:
            rval = pwbem.Sint32(arg.errno)
        out_params = []
        return (rval, out_params)

        
    class Values(object):
        class DetailedStatus(object):
            Not_Available = pywbem.Uint16(0)
            No_Additional_Information = pywbem.Uint16(1)
            Stressed = pywbem.Uint16(2)
            Predictive_Failure = pywbem.Uint16(3)
            Non_Recoverable_Error = pywbem.Uint16(4)
            Supporting_Entity_in_Error = pywbem.Uint16(5)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class RequestedState(object):
            Unknown = pywbem.Uint16(0)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class HealthState(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(5)
            Degraded_Warning = pywbem.Uint16(10)
            Minor_failure = pywbem.Uint16(15)
            Major_failure = pywbem.Uint16(20)
            Critical_failure = pywbem.Uint16(25)
            Non_recoverable_error = pywbem.Uint16(30)
            # DMTF_Reserved = ..

        class TransitioningToState(object):
            Unknown = pywbem.Uint16(0)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            No_Change = pywbem.Uint16(5)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)
            Not_Applicable = pywbem.Uint16(12)

        class EnabledDefault(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            No_Default = pywbem.Uint16(7)
            Quiesce = pywbem.Uint16(9)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 32768..65535

        class EnabledState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shutting_Down = pywbem.Uint16(4)
            Not_Applicable = pywbem.Uint16(5)
            Enabled_but_Offline = pywbem.Uint16(6)
            In_Test = pywbem.Uint16(7)
            Deferred = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Starting = pywbem.Uint16(10)
            # DMTF_Reserved = 11..32767
            # Vendor_Reserved = 32768..65535

        class ExecutionState(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            Ready = pywbem.Uint16(2)
            Running = pywbem.Uint16(3)
            Blocked = pywbem.Uint16(4)
            Suspended_Blocked = pywbem.Uint16(5)
            Suspended_Ready = pywbem.Uint16(6)
            Terminated = pywbem.Uint16(7)
            Stopped = pywbem.Uint16(8)
            Growing = pywbem.Uint16(9)
            Ready_But_Relinquished_Processor = pywbem.Uint16(10)
            Hung = pywbem.Uint16(11)

        class AvailableRequestedStates(object):
            Enabled = pywbem.Uint16(2)
            Disabled = pywbem.Uint16(3)
            Shut_Down = pywbem.Uint16(4)
            Offline = pywbem.Uint16(6)
            Test = pywbem.Uint16(7)
            Defer = pywbem.Uint16(8)
            Quiesce = pywbem.Uint16(9)
            Reboot = pywbem.Uint16(10)
            Reset = pywbem.Uint16(11)

        class Status(object):
            OK = 'OK'
            Error = 'Error'
            Degraded = 'Degraded'
            Unknown = 'Unknown'
            Pred_Fail = 'Pred Fail'
            Starting = 'Starting'
            Stopping = 'Stopping'
            Service = 'Service'
            Stressed = 'Stressed'
            NonRecover = 'NonRecover'
            No_Contact = 'No Contact'
            Lost_Comm = 'Lost Comm'
            Stopped = 'Stopped'

        class CommunicationStatus(object):
            Unknown = pywbem.Uint16(0)
            Not_Available = pywbem.Uint16(1)
            Communication_OK = pywbem.Uint16(2)
            Lost_Communication = pywbem.Uint16(3)
            No_Contact = pywbem.Uint16(4)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class OperatingStatus(object):
            Unknown = pywbem.Uint16(0)
            Not_Available = pywbem.Uint16(1)
            Servicing = pywbem.Uint16(2)
            Starting = pywbem.Uint16(3)
            Stopping = pywbem.Uint16(4)
            Stopped = pywbem.Uint16(5)
            Aborted = pywbem.Uint16(6)
            Dormant = pywbem.Uint16(7)
            Completed = pywbem.Uint16(8)
            Migrating = pywbem.Uint16(9)
            Emigrating = pywbem.Uint16(10)
            Immigrating = pywbem.Uint16(11)
            Snapshotting = pywbem.Uint16(12)
            Shutting_Down = pywbem.Uint16(13)
            In_Test = pywbem.Uint16(14)
            Transitioning = pywbem.Uint16(15)
            In_Service = pywbem.Uint16(16)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class RequestStateChange(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown_or_Unspecified_Error = pywbem.Uint32(2)
            Cannot_complete_within_Timeout_Period = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Invalid_State_Transition = pywbem.Uint32(4097)
            Use_of_Timeout_Parameter_Not_Supported = pywbem.Uint32(4098)
            Busy = pywbem.Uint32(4099)
            # Method_Reserved = 4100..32767
            # Vendor_Specific = 32768..65535
            class RequestedState(object):
                Enabled = pywbem.Uint16(2)
                Disabled = pywbem.Uint16(3)
                Shut_Down = pywbem.Uint16(4)
                Offline = pywbem.Uint16(6)
                Test = pywbem.Uint16(7)
                Defer = pywbem.Uint16(8)
                Quiesce = pywbem.Uint16(9)
                Reboot = pywbem.Uint16(10)
                Reset = pywbem.Uint16(11)
                # DMTF_Reserved = ..
                # Vendor_Reserved = 32768..65535

        class kill(object):
            class signal(object):
                SIGHUP = pywbem.Uint16(1)
                SIGKILL = pywbem.Uint16(9)
                SIGTERM = pywbem.Uint16(15)

        class PrimaryStatus(object):
            Unknown = pywbem.Uint16(0)
            OK = pywbem.Uint16(1)
            Degraded = pywbem.Uint16(2)
            Error = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

## end of class Py_UnixProcessProvider

def get_providers(env): 
    py_unixprocess_prov = Py_UnixProcessProvider(env)  
    return {'Py_UnixProcess': py_unixprocess_prov} 




"""Python Provider for TestAssoc

Instruments:
- TestAssoc_User (instance provider)
- TestAssoc_Group (instance provider)
- TestAssoc_MemberOfGroup (instance and association provider)

"""

import pywbem
import pwd
import grp
from pywbem.cim_provider2 import CIMProvider2

def get_user_instance(uid, model, keys_only):
    try:
        pwinfo = pwd.getpwuid(uid)
        #model['UserID'] = pywbem.Uint32(uid)
        model['UserID'] = str(uid)
        if hasattr(model, 'path'):
            model.path['UserID'] = str(uid)
        if keys_only:
            return model
        model['UserName'] = pwinfo[0]
        model['LoginShell'] = pwinfo[6]
        model['HomeDirectory'] = pwinfo[5]
        return model
    except KeyError:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
def get_group_instance(gid, model, keys_only):
    try:
        grinfo = grp.getgrgid(gid)
        #model['GroupID'] = pywbem.Uint32(gid)
        model['GroupID'] = str(gid)
        if hasattr(model, 'path'):
            model.path['GroupID'] = str(gid)
        if keys_only:
            return model
        model['GroupName'] = grinfo[0]
        return model
    except KeyError:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
def is_primary_user(uid, gid):
    pwinfo = pwd.getpwuid(uid)
    return gid == pwinfo[3]
    
def is_user_in_group(uid, gid):
    pwinfo = pwd.getpwuid(uid)
    grinfo = grp.getgrgid(gid)
    return pwinfo[0] in grinfo[3]
        
def get_assoc_instance(uid, gid, model, keys_only):
    try:
        isPrimary = is_primary_user(uid, gid)
        if not isPrimary and not is_user_in_group(uid, gid):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        model['Dependent'] = get_user_instance(uid, model.path['Dependent'], keys_only)
        model['Antecedent'] = get_group_instance(gid, model.path['Antecedent'], keys_only)
        if keys_only:
            return model
        model['isPrimaryGroup'] = isPrimary
        return model
    except KeyError:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)

class TestAssoc_User(CIMProvider2):
    
    def __init__(self, env):
        self._logger = env.get_logger()
        
    def get_instance(self, env, model, inst=None):
        try:
            uid = model.path['UserID']
            uid = int(uid)
            return get_user_instance(uid, model, False)
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
    def enum_instances(self, env, model, keys_only):
        self._logger.log_debug("%s:  enum_instances called for class %s" % (self.__class__.__name__.upper(), model.classname))
        for pwent in pwd.getpwall():
            yield get_user_instance(pwent[2], model, keys_only)
        
    def set_instance(self, env, instance, modify_existing):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    
    
class TestAssoc_Group(CIMProvider2):
    
    def __init__(self, env):
        self._logger = env.get_logger()
        
    def get_instance(self, env, model, inst=None):
        try:
            gid = model.path['GroupID']
            gid = int(gid)
            return get_group_instance(gid, model, False)
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
    def enum_instances(self, env, model, keys_only):
        for grent in grp.getgrall():
            yield get_group_instance(grent[2], model, keys_only)
        
    def set_instance(self, env, instance, modify_existing):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    
class TestAssoc_MemberOfGroup(CIMProvider2):
    
    def __init__(self, env):
        self._logger = env.get_logger()
        
    def get_instance(self, env, model, inst=None):
        try:
            uid = model.path['Dependent']['UserID']
            uid = int(uid)
            gid = model.path['Antecedent']['GroupID']
            gid = int(gid)
            return get_assoc_instance(uid, gid, model, False)
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
    def enum_instances(self, env, model, keys_only):
        self._logger.log_debug("\n%s:  enum_instances called for class %s" % (self.__class__.__name__.upper(), model.classname))
        for pwent in pwd.getpwall():
            user_cin = pywbem.CIMInstanceName('TestAssoc_User',
                    namespace=model.path.namespace)
            group_cin = pywbem.CIMInstanceName('TestAssoc_Group',
                    namespace=model.path.namespace)
            model['Dependent'] = get_user_instance(pwent[2], user_cin, True)
            model.path['Dependent'] = get_user_instance(pwent[2], 
                    user_cin, True)
            model['Antecedent'] = get_group_instance(pwent[3], group_cin, True)
            model.path['Antecedent'] = get_group_instance(pwent[3], 
                    group_cin, True)
            if not keys_only:
                model['isPrimaryGroup'] = True
            yield model
            for grent in grp.getgrall():
                if pwent[0] in grent[3]:
                    model['Antecedent'] = get_group_instance(grent[2], 
                            group_cin, True)
                    model.path['Antecedent'] = get_group_instance(grent[2], 
                            group_cin, True)
                    if not keys_only:
                        model['isPrimaryGroup'] = False
                    yield model
        
    def set_instance(self, env, instance, modify_existing):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)
        
    def delete_instance(self, env, instance_name):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    def references(self, env, object_name, model, 
            result_class_name, role, result_role, keys_only):
        self._logger.log_debug("\n%s:  References called for class %s" % (self.__class__.__name__.upper(), object_name))
        model.path.keybindings['antecedent'] = None
        model.path.keybindings['dependent'] = None
        if object_name.classname.lower() == 'testassoc_user':
            if role and role.lower() == 'antecedent':
                return
            if result_role and result_role.lower() == 'dependent':
                return
            if result_class_name and \
                    result_class_name.lower() != 'testassoc_group':
                return
            model['Dependent'] = object_name
            cn = pywbem.CIMInstanceName('TestAssoc_Group',
                    namespace=object_name.namespace)
            uid = model['Dependent']['UserID']
            uid = int(uid)
            pwinfo = pwd.getpwuid(uid)
            model['Antecedent'] = get_group_instance(pwinfo[3], cn, True)
            if not keys_only:
                model['isPrimaryGroup'] = True
            yield model
            for grent in grp.getgrall():
                if pwinfo[0] in grent[3]:
                    model['Antecedent'] = get_group_instance(grent[2], cn, True)
                    if not keys_only:
                        model['isPrimaryGroup'] = False
                    yield model
        if object_name.classname.lower() == 'testassoc_group':
            if role and role.lower() == 'dependent':
                return
            if result_role and result_role.lower() == 'antecedent':
                return
            if result_class_name and \
                    result_class_name.lower() != 'testassoc_user':
                return
            model['Antecedent'] = object_name
            cn = pywbem.CIMInstanceName('TestAssoc_User',
                    namespace=object_name.namespace)
            gid = model['Antecedent']['GroupID']
            gid = int(gid)
            grinfo = grp.getgrgid(gid)
            for member_name in grinfo[3]:
                pwinfo = pwd.getpwnam(member_name)
                model['Dependent'] = get_user_instance(pwinfo[2], cn, True)
                if not keys_only:
                    model['isPrimaryGroup'] = False
                yield model
            for pwent in pwd.getpwall():
                if pwent[3] == grinfo[2]:
                    model['Dependent'] = get_user_instance(pwent[2], cn, True)
                    if not keys_only:
                        model['isPrimaryGroup'] = True
                    yield model
        
    
def get_providers(env):
    user_prov = TestAssoc_User(env)
    group_prov = TestAssoc_Group(env)
    mog_prov = TestAssoc_MemberOfGroup(env)
    return { 'TestAssoc_User' : user_prov,
        'TestAssoc_Group' : group_prov,
        'TestAssoc_MemberOfGroup' : mog_prov }



"""Python Provider for TestAtom

Instruments the CIM class TestAtom

"""

import pywbem
import sys
from pywbem.cim_provider2 import CIMProvider2

class TestAtomProvider(CIMProvider2):
    """Instrument the CIM class TestAtom 

    Model an atom, For use with CIMOM and PyWBEM Provider
    
    """
    def __init__ (self, env):
        logger = env.get_logger()
        logger.log_debug('Initializing provider %s from %s' \
                % (self.__class__.__name__, __file__))
        self.storage = {}



    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The 
            key properties are set on this instance to correspond to the 
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the 
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the 
            values, and the instance will be filtered for you. 
        cim_class -- The pywbem.CIMClass

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        
        logger = env.get_logger()
        logger.log_debug('Entering %s.get_instance()' \
                % self.__class__.__name__)
       
        try:
            #logger.log_debug("**** GET_INSTANCE model[name]: %s ****" % model['Name'])
            #print "**** GET_INSTANCE model[name]: %s ****" % str(model['Name'])
            #if model['Name'] in self.storage.keys():
            inst = self.storage[model['Name']]
            #else:
                #print "This is not working.... ******* FIX ME"

        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        #print " **** Setting Model Properties: ****"
        for k, v in inst.properties.items():
            model[k] = v

        #return inst
        return model


    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method. 
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.  
            The properties of the model are already filtered according to 
            the PropertyList from the request.  Only properties present in 
            the model need to be given values.  If you prefer, you can 
            always set all of the values, and the instance will be filtered 
            for you. 
        cim_class -- The pywbem.CIMClass
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.enum_instances()' \
                % self.__class__.__name__)
        #for atom in self.storage.keys():
            #print "Key = %s " %str(atom)

        for key in self.storage.keys():
            #print "***** HELLO ***** "
            #logger.log_debug("************ ENUM_INSTANCES ********")
            #logger.log_debug(" **** model['Name'] = %s ****" % key)
            #print "************ ENUM_INSTANCES ********"
            #print " **** model['Name'] = %s ****" % key
            model['Name'] = key
            model.path['Name'] = key
            try:
                yield self.get_instance(env, model)
            except pywbem.CIMError, (num, msg):
                if num not in (pywbem.CIM_ERR_NOT_FOUND, 
                               pywbem.CIM_ERR_ACCESS_DENIED):
                    raise

    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing 
            instance, the properties on this instance have been filtered by 
            the PropertyList from the request.
        previous_instance -- The previous pywbem.CIMInstance if modifying 
            an existing instance.  None if creating a new instance. 
        cim_class -- The pywbem.CIMClass

        Return the new instance.  The keys must be set on the new instance. 

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only 
            valid if previous_instance is None, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid 
            if previous_instance is not None, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """

        logger = env.get_logger()
        logger.log_debug('Entering %s.set_instance()' \
                % self.__class__.__name__)

        if modify_existing:
            if instance['Name'] not in self.storage:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
            inst = self.storage[instance.path['Name']] 
            inst.properties.update(instance.properties)
            #logger.log_debug("***** Updating stuff :%s *****" % instance.properties)

        else:
            if instance.path['Name'] in self.storage:
                    raise pywbem.CIMError(pywbem.CIM_ERR_ALREADY_EXISTS)
            else:
                #Creating a new instance
                #print "Copying Instance"
                #print "Instance name: %s"%str(instance['Name'])
                #for key in instance.properties.keys():
                    #print "key=%s"%str(key)
                self.storage[instance.path['Name']] = instance.copy()

        instance.path['Name'] = instance['Name']
        return instance

    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance 
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized 
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified 
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM 
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """ 
        logger = env.get_logger()
        logger.log_debug('Entering %s.delete_instance()' \
                % self.__class__.__name__)
        try:
            del self.storage[instance_name['Name']]
        except KeyError:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND)
        
## end of class TestAtomProvider

def get_providers(env): 
    testatom_prov = TestAtomProvider(env)  
    return {'Test_Atom': testatom_prov} 
